using System;
using System.Collections;
using System.Xml;
using AspectWeaver.Advices;

namespace AspectWeaver
{
	public class AdviceFactory
	{
		private XmlDocument baseDoc;
		private Hashtable aspectDocs = new Hashtable();
		private WeaveLogger logger;

		public AdviceFactory(XmlDocument baseDoc, Hashtable aspectDocs, WeaveLogger logger)
		{
			this.baseDoc = baseDoc;
			this.aspectDocs = aspectDocs;
			this.logger = logger;
		}

		public BaseAdvice CreateAdvice(XmlNode adviceNode)
		{
			switch (adviceNode.LocalName)
			{
				case "Delete":
					return new Delete(adviceNode, baseDoc, logger);
				case "SetBaseType":
					return new SetBaseType(adviceNode, baseDoc, logger);
				case "ImplementInterface":
					return new ImplementInterface(adviceNode, baseDoc, logger);
				case "MakeSealed":
					return new MakeSealed(adviceNode, baseDoc, logger);
				case "MakeUnsealed":
					return new MakeUnsealed(adviceNode, baseDoc, logger);
				case "MakeSerializable":
					return new MakeSerializable(adviceNode, baseDoc, logger);
				case "Introduce":
					return new Introduce(adviceNode, baseDoc, aspectDocs, logger);
				case "BeforeConstructorCall":
					return new BeforeConstructorCall(adviceNode, baseDoc, aspectDocs, logger);
				case "AfterConstructorCall":
					return new AfterConstructorCall(adviceNode, baseDoc, aspectDocs, logger);
				case "BeforeMethodCall":
					return new BeforeMethodCall(adviceNode, baseDoc, aspectDocs, logger);
				case "AfterMethodCall":
					return new AfterMethodCall(adviceNode, baseDoc, aspectDocs, logger);
				case "InlineAtStart":
					return new InlineAtStart(adviceNode, baseDoc, aspectDocs, logger);
				case "InlineBeforeReturn":
					return new InlineBeforeReturn(adviceNode, baseDoc, aspectDocs, logger);
				case "AroundBody":
					return new AroundBody(adviceNode, baseDoc, aspectDocs, logger);
				default:
					return new BaseAdvice(adviceNode, baseDoc, aspectDocs, logger);
			}
		}
	}
}
