using System;
using System.Collections;
using System.Reflection;
using System.Xml;

namespace AspectWeaver.Advices
{
	public class AfterConstructorCall : BaseAdvice
	{
		public AfterConstructorCall(XmlNode adviceNode, XmlDocument baseDoc, Hashtable aspectDocs, WeaveLogger logger) : base(adviceNode, baseDoc, aspectDocs, logger)
		{
		}

		public override void DoWeave()
		{
			foreach (XmlNode targetNode in GetTargetNodes())
			{
				XmlNodeList afterConstructorCallNodes = baseDoc.SelectNodes("//Instruction[@code='newobj' and @operand = '" + targetNode.Attributes["signature"].InnerXml + "']");
				foreach (XmlNode afterConstructorCallNode in afterConstructorCallNodes)
				{
					AddAffectedElementLog(Utils.GetElementDesc(afterConstructorCallNode.ParentNode));

					//get code to be added first
					XmlNodeList codeNodes = Utils.CreateCloneCodeNodes(((XmlDocument)aspectDocs[adviceNode.SelectSingleNode("Code").Attributes["aspectAssemblyUniqueName"].InnerXml]).SelectNodes(adviceNode.SelectSingleNode("Code").Attributes["aspectXPath"].InnerXml + "/Instruction"));

					Utils.ReplaceGetArgumentsWithLocalCode(codeNodes, afterConstructorCallNode.ParentNode);
					Utils.IntroduceLocalVariables(codeNodes, afterConstructorCallNode.ParentNode);
					Utils.ReplaceGetContextInfoWithLocalCode(codeNodes, afterConstructorCallNode.ParentNode);

					//get ref code node
					XmlNode refNode = afterConstructorCallNode.NextSibling;
					if (refNode.Attributes["code"].InnerXml == "box")
					{
						refNode = refNode.NextSibling;
					}
					refNode = refNode.NextSibling;

					//do add code
					Utils.InsertCodeNodesBefore(codeNodes, refNode);
				}
			}
		
			SetStatusLog("weaved successfully");
		}
	}
}
