using System;
using System.Collections;
using System.Reflection;
using System.Xml;

namespace AspectWeaver.Advices
{
	public class AfterMethodCall : BaseAdvice
	{
		public AfterMethodCall(XmlNode adviceNode, XmlDocument baseDoc, Hashtable aspectDocs, WeaveLogger logger) : base(adviceNode, baseDoc, aspectDocs, logger)
		{
		}

		public override void DoWeave()
		{
			foreach(XmlNode targetNode in GetTargetNodes())
			{
				XmlNodeList afterMethodCallNodes = baseDoc.SelectNodes("//Instruction[@operand = '" + targetNode.Attributes["signature"].InnerXml + "' and starts-with(@code, 'call')]");
				foreach (XmlNode afterMethodCallNode in afterMethodCallNodes)
				{
					AddAffectedElementLog(Utils.GetElementDesc(afterMethodCallNode.ParentNode));

					//get code to be added first
					XmlNodeList codeNodes = Utils.CreateCloneCodeNodes(((XmlDocument)aspectDocs[adviceNode.SelectSingleNode("Code").Attributes["aspectAssemblyUniqueName"].InnerXml]).SelectNodes(adviceNode.SelectSingleNode("Code").Attributes["aspectXPath"].InnerXml + "/Instruction"));

					Utils.ReplaceGetArgumentsWithLocalCode(codeNodes, afterMethodCallNode.ParentNode);
					Utils.IntroduceLocalVariables(codeNodes, afterMethodCallNode.ParentNode);
					Utils.ReplaceGetContextInfoWithLocalCode(codeNodes, afterMethodCallNode.ParentNode);

					//get ref code node
					XmlNode refNode = afterMethodCallNode.NextSibling;
					if (refNode.Attributes["code"].InnerXml == "box")
					{
						refNode = refNode.NextSibling;
					}

					//do add code
					Utils.InsertCodeNodesBefore(codeNodes, refNode);
				}
			}
	
			SetStatusLog("weaved successfully");
		}
	}
}
