using System;
using System.Collections;
using System.Reflection;
using System.Xml;

namespace AspectWeaver.Advices
{
	public class BaseAdvice
	{
		protected XmlNode adviceNode;
		protected XmlDocument baseDoc;
		protected Hashtable aspectDocs;
		private WeaveLogger logger;
		private XmlNode logNode;

		public BaseAdvice()
		{
		}

		public BaseAdvice(XmlNode adviceNode, XmlDocument baseDoc, WeaveLogger logger) : this()
		{
			this.adviceNode = adviceNode;
			this.logger = logger;
			this.baseDoc = baseDoc;
			logNode = logger.CreateLogNode(adviceNode);
		}

		public BaseAdvice(XmlNode adviceNode, XmlDocument baseDoc, Hashtable aspectDocs, WeaveLogger logger) : this(adviceNode, baseDoc, logger)
		{
			this.aspectDocs = aspectDocs;
		}

		protected void AddAffectedElementLog(string desc)
		{
			if (logNode.SelectSingleNode("AffectedElements") == null)
			{
				logNode.AppendChild(logger.CreateAffectedElementsNode());
			}

			bool isAlreadyExisted = false;
			XmlNodeList existedLogNodes = logNode.SelectNodes("AffectedElements/AffectedElement");
			foreach (XmlNode existedLogNode in existedLogNodes)
			{
				if (existedLogNode.InnerXml == desc)
				{
					isAlreadyExisted = true;
					break;
				}
			}

			if (!isAlreadyExisted)
			{
				logNode.SelectSingleNode("AffectedElements").AppendChild(logger.CreateAffectedElementNode(desc));
			}
		}

		protected void SetStatusLog(string status)
		{
			logNode.Attributes["status"].InnerXml = status;
		}

		protected XmlNodeList GetTargetNodes()
		{
			return baseDoc.SelectNodes(adviceNode.SelectSingleNode("Pointcut").Attributes["targetXPath"].InnerXml);
		}
		public virtual void DoWeave()
		{
			SetStatusLog("skipped - not supported advice");
		}
	}
}
