using System;
using System.Collections;
using System.Reflection;
using System.Xml;

namespace AspectWeaver.Advices
{
	public class BeforeConstructorCall : BaseAdvice
	{
		public BeforeConstructorCall(XmlNode adviceNode, XmlDocument baseDoc, Hashtable aspectDocs, WeaveLogger logger) : base(adviceNode, baseDoc, aspectDocs, logger)
		{
		}

		public override void DoWeave()
		{
			foreach (XmlNode targetNode in GetTargetNodes())
			{
				XmlNodeList beforeConstructorCallNodes = baseDoc.SelectNodes("//Instruction[@code='newobj' and @operand = '" + targetNode.Attributes["signature"].InnerXml + "']");
				foreach (XmlNode beforeConstructorCallNode in beforeConstructorCallNodes)
				{
					AddAffectedElementLog(Utils.GetElementDesc(beforeConstructorCallNode.ParentNode));

					//get code to be added first
					XmlNodeList codeNodes = Utils.CreateCloneCodeNodes(((XmlDocument)aspectDocs[adviceNode.SelectSingleNode("Code").Attributes["aspectAssemblyUniqueName"].InnerXml]).SelectNodes(adviceNode.SelectSingleNode("Code").Attributes["aspectXPath"].InnerXml + "/Instruction"));

					Utils.ReplaceGetArgumentsWithLocalCode(codeNodes, beforeConstructorCallNode.ParentNode);
					Utils.IntroduceLocalVariables(codeNodes, beforeConstructorCallNode.ParentNode);
					Utils.ReplaceGetContextInfoWithLocalCode(codeNodes, beforeConstructorCallNode.ParentNode);

					//get ref node
					XmlNode refNode = Utils.GetRefNodeBeforeConstructorOrMethodCall(beforeConstructorCallNode.PreviousSibling, targetNode.SelectNodes("Parameter").Count);

					//do add code
					Utils.InsertCodeNodesBefore(codeNodes, refNode);
				}
			}
		
			SetStatusLog("weaved successfully");
		}
	}
}
