using System;
using System.Collections;
using System.Reflection;
using System.Xml;

namespace AspectWeaver.Advices
{
	public class BeforeMethodCall : BaseAdvice
	{
		public BeforeMethodCall(XmlNode adviceNode, XmlDocument baseDoc, Hashtable aspectDocs, WeaveLogger logger) : base(adviceNode, baseDoc, aspectDocs, logger)
		{
		}

		public override void DoWeave()
		{
			foreach(XmlNode targetNode in GetTargetNodes())
			{
				XmlNodeList beforeMethodCallNodes = baseDoc.SelectNodes("//Instruction[@operand = '" + targetNode.Attributes["signature"].InnerXml + "' and starts-with(@code, 'call')]");
				foreach (XmlNode beforeMethodCallNode in beforeMethodCallNodes)
				{
					AddAffectedElementLog(Utils.GetElementDesc(beforeMethodCallNode.ParentNode));

					//get code to be added first
					XmlNodeList codeNodes = Utils.CreateCloneCodeNodes(((XmlDocument)aspectDocs[adviceNode.SelectSingleNode("Code").Attributes["aspectAssemblyUniqueName"].InnerXml]).SelectNodes(adviceNode.SelectSingleNode("Code").Attributes["aspectXPath"].InnerXml + "/Instruction"));

					Utils.ReplaceGetArgumentsWithLocalCode(codeNodes, beforeMethodCallNode.ParentNode);
					Utils.IntroduceLocalVariables(codeNodes, beforeMethodCallNode.ParentNode);
					Utils.ReplaceGetContextInfoWithLocalCode(codeNodes, beforeMethodCallNode.ParentNode);

					//get ref code node
					XmlNode refNode = Utils.GetRefNodeBeforeConstructorOrMethodCall(beforeMethodCallNode.PreviousSibling, (targetNode.Attributes["isStatic"].InnerXml == "True" ? 0 : 1) + targetNode.SelectNodes("Parameter").Count);

					//do add code
					Utils.InsertCodeNodesBefore(codeNodes, refNode);
				}
			}
		
			SetStatusLog("weaved successfully");
		}
	}
}
