using System;
using System.Collections;
using System.Reflection;
using System.Xml;

namespace AspectWeaver.Advices
{
	public class InlineAtStart : BaseAdvice
	{
		public InlineAtStart(XmlNode adviceNode, XmlDocument baseDoc, Hashtable aspectDocs, WeaveLogger logger) : base(adviceNode, baseDoc, aspectDocs, logger)
		{
		}

		public override void DoWeave()
		{
			foreach(XmlNode targetNode in GetTargetNodes())
			{
				AddAffectedElementLog(Utils.GetElementDesc(targetNode));

				//get code to be added first
				XmlNodeList codeNodes = Utils.CreateCloneCodeNodes(((XmlDocument)aspectDocs[adviceNode.SelectSingleNode("Code").Attributes["aspectAssemblyUniqueName"].InnerXml]).SelectNodes(adviceNode.SelectSingleNode("Code").Attributes["aspectXPath"].InnerXml + "/Instruction"));

				Utils.ReplaceGetArgumentsWithLocalCode(codeNodes, targetNode);
				Utils.IntroduceLocalVariables(codeNodes, targetNode);
				Utils.ReplaceGetContextInfoWithLocalCode(codeNodes, targetNode);

				//get ref code node
				XmlNode refNode = targetNode.SelectSingleNode("Instruction[@offset = 0]");

				if (targetNode.LocalName == "Constructor")
				{
					//skip call base()
					while (refNode != null && refNode.Attributes["code"].InnerXml != "call")
					{
						refNode = refNode.NextSibling;
					}
					refNode = refNode.NextSibling;
				}

				//do add code inline at start
				Utils.InsertCodeNodesBefore(codeNodes, refNode);
			}
		
			SetStatusLog("weaved successfully");
		}
	}
}
