using System;
using System.Collections;
using System.Reflection;
using System.Xml;

namespace AspectWeaver.Advices
{
	public class Introduce : BaseAdvice
	{
		private void ReplaceGetContextInfoAndGetArgumentsCodeWithLocalCode(XmlNode sourceMethodNode, XmlNode targetMethodNode)
		{
			XmlNodeList cloneCodeNodes = sourceMethodNode.ChildNodes;
			Utils.ReplaceGetContextInfoWithLocalCode(cloneCodeNodes, targetMethodNode);
			Utils.ReplaceGetArgumentsWithLocalCode(cloneCodeNodes, targetMethodNode);
		}

		public Introduce(XmlNode adviceNode, XmlDocument baseDoc, Hashtable aspectDocs, WeaveLogger logger) : base(adviceNode, baseDoc, aspectDocs, logger)
		{
		}

		public override void DoWeave()
		{
			foreach (XmlNode nodeToIntroduce in GetTargetNodes())
			{
				AddAffectedElementLog(Utils.GetElementDesc(nodeToIntroduce));

				XmlNodeList nodesToBeIntroduced = ((XmlDocument)aspectDocs[adviceNode.SelectSingleNode("Code").Attributes["aspectAssemblyUniqueName"].InnerXml]).SelectNodes(adviceNode.SelectSingleNode("Code").Attributes["aspectXPath"].InnerXml);
				foreach (XmlNode nodeToBeIntroduced in nodesToBeIntroduced)
				{
					XmlNode newNode = Utils.CreateCloneNode(baseDoc, nodeToBeIntroduced);

					// clear custom advice configuration attributes
					Utils.RemoveNodesFromTheirParentNode(newNode.SelectNodes(".//Attribute[starts-with(@type, 'AspectWeaver.Aspects.')]"));

					// replace GetContextInfo and GetArguments codes for each Method in nodeToBeIntroduced
					foreach (XmlNode methodNode in nodeToBeIntroduced.SelectNodes("Method"))
					{
						ReplaceGetContextInfoAndGetArgumentsCodeWithLocalCode(methodNode, newNode.SelectSingleNode(string.Format(".//Method[@signature = '{0}']", methodNode.Attributes["signature"].InnerXml)));
					}

					// replace GetContextInfo and GetArguments codes if current node is a Method
					if (nodeToBeIntroduced.LocalName == "Method")
					{
						ReplaceGetContextInfoAndGetArgumentsCodeWithLocalCode(nodeToBeIntroduced, newNode);
					}

					nodeToIntroduce.AppendChild(newNode);
				}
			}
		
			SetStatusLog("weaved successfully");
		}

	}
}
