using System;
using System.Collections;
using System.Xml;
using System.Reflection;

namespace AspectWeaver
{
	/// <summary>
	/// Summary description for ConfigurationVerifier.
	/// </summary>
	public class ConfigurationVerifier
	{
		private XmlDocument doc;
		private Hashtable aspectIlmlDocs = new Hashtable();

		public ConfigurationVerifier(string configPath)
		{
			doc = new XmlDocument();
			doc.Load(configPath);
		}

		public XmlDocument DoVerify()
		{
			try
			{
				XmlNode advicesNode = doc.SelectSingleNode("//Advices");
				if (advicesNode == null)
				{
					advicesNode = doc.CreateElement("Advices");
					doc.SelectSingleNode("/Configuration").InsertBefore(advicesNode, doc.SelectSingleNode("//AdviceFiles"));
				}
				
				// load all advice files to Advices node first
				XmlNodeList adviceFiles = doc.SelectNodes("//AdviceFile");

				if (adviceFiles != null && adviceFiles.Count > 0)
				{
					foreach (XmlNode adviceFile in adviceFiles)
					{
						XmlDocument adviceFileDoc = new XmlDocument();
						adviceFileDoc.Load(AspectWeaver.MainClass.WorkDir + adviceFile.InnerXml);
						// add Advice nodes in adviceFileDoc to doc's Advices node
						foreach (XmlNode advice in adviceFileDoc.SelectNodes("/Advices/*"))
						{
							XmlNode newAdvice = doc.CreateElement(advice.Name);
							newAdvice.InnerXml = advice.InnerXml;
							advicesNode.AppendChild(newAdvice);
						}
					}

					doc.SelectSingleNode("/Configuration").RemoveChild(doc.SelectSingleNode("//AdviceFiles"));
				}

				//create tmp dir
				if (!System.IO.Directory.Exists(AspectWeaver.MainClass.WorkDir + MainClass.TEMP_DIR))
				{
					System.IO.Directory.CreateDirectory(AspectWeaver.MainClass.WorkDir + MainClass.TEMP_DIR);
				}

				Ilml.IIlmlParser parser = Ilml.IlmlParserFactory.GetParser();

				// load aspect assembly ilml docs
				foreach (XmlNode aspectAsemblyNode in doc.SelectNodes("//AspectAssembly"))
				{
					XmlDocument aspectAssemblyDoc = new XmlDocument();
					if (aspectAsemblyNode.Attributes["path"].InnerXml.Length > 0)
					{
						string outputFilePath = AspectWeaver.MainClass.WorkDir + MainClass.TEMP_DIR + "\\" + aspectAsemblyNode.Attributes["uniqueName"].InnerXml + ".ilml.xml";
						parser.Disassemble(new System.IO.FileInfo(AspectWeaver.MainClass.WorkDir + aspectAsemblyNode.Attributes["path"].InnerXml), new System.IO.FileInfo(outputFilePath));
						aspectAssemblyDoc.Load(outputFilePath);
					}
					else
					{
						throw new Exception("/Configuration/AspectAssembly not exists or error!");
					}
					aspectIlmlDocs.Add(aspectAsemblyNode.Attributes["uniqueName"].InnerXml, aspectAssemblyDoc);
				}

				//load all advice attributes to Advices
				foreach (string aspectUniqueName in aspectIlmlDocs.Keys)
				{
					XmlDocument aspectDoc = (XmlDocument)aspectIlmlDocs[aspectUniqueName];
					XmlNodeList nodes = aspectDoc.SelectNodes("//Attribute[starts-with(@type, 'AspectWeaver.Aspects.')]");
					foreach (XmlNode node in nodes)
					{
						string attrName = node.Attributes["type"].InnerXml.Substring("AspectWeaver.Aspects.".Length);
						if (attrName.EndsWith("Attribute"))
						{
							attrName = attrName.Substring(0, attrName.Length - "Attribute".Length);
						}
						XmlNode newAdvice = doc.CreateElement(attrName);
						if (node.ParentNode.LocalName == "Type")
						{
							newAdvice.InnerXml = string.Format("<Pointcut targetXPath=\"//Module\" /><Code aspectAssemblyUniqueName=\"{1}\" aspectXPath=\"//{2}[@fullName='{3}']\" />", System.Text.Encoding.UTF8.GetString(Convert.FromBase64String(node.Attributes["data"].InnerXml)), aspectUniqueName, node.ParentNode.LocalName, node.ParentNode.Attributes["fullName"].InnerXml);
						}
						else
						{
							string aspectXPath = System.Text.Encoding.UTF8.GetString(Convert.FromBase64String(node.Attributes["data"].InnerXml));
							aspectXPath = aspectXPath.Substring(aspectXPath.IndexOf("//")).Trim('\0');
							if (node.ParentNode.Attributes["signature"] != null)
							{
								newAdvice.InnerXml = string.Format("<Pointcut targetXPath=\"{0}\" /><Code aspectAssemblyUniqueName=\"{1}\" aspectXPath=\"//{2}[@fullName='{3}']/{4}[@signature='{5}']\" />", aspectXPath, aspectUniqueName, node.ParentNode.ParentNode.LocalName, node.ParentNode.ParentNode.Attributes["fullName"].InnerXml, node.ParentNode.LocalName, node.ParentNode.Attributes["signature"].InnerXml);
							}
							else if (node.ParentNode.Attributes["fullName"] != null)
							{
								newAdvice.InnerXml = string.Format("<Pointcut targetXPath=\"{0}\" /><Code aspectAssemblyUniqueName=\"{1}\" aspectXPath=\"//{2}[@fullName='{3}']/{4}[@fullName='{5}']\" />", aspectXPath, aspectUniqueName, node.ParentNode.ParentNode.LocalName, node.ParentNode.ParentNode.Attributes["fullName"].InnerXml, node.ParentNode.LocalName, node.ParentNode.Attributes["fullName"].InnerXml);
							}
							else
							{
								newAdvice.InnerXml = string.Format("<Pointcut targetXPath=\"{0}\" /><Code aspectAssemblyUniqueName=\"{1}\" aspectXPath=\"//{2}[@fullName='{3}']/{4}[@name='{5}']\" />", aspectXPath, aspectUniqueName, node.ParentNode.ParentNode.LocalName, node.ParentNode.ParentNode.Attributes["fullName"].InnerXml, node.ParentNode.LocalName, node.ParentNode.Attributes["name"].InnerXml);
							}
						}
						advicesNode.AppendChild(newAdvice);
					}
				}																											
				
				// add additional verifying in later version here
			}
			catch(Exception e)
			{
				Console.Write(e.ToString() + "\n\n");
				return null;
			}

			return doc;
		}
	}
}
