using System;
using System.Xml;

namespace AspectWeaver
{
	/// <summary>
	/// Summary description for Main.
	/// </summary>
	public class MainClass
	{
		public const string TEMP_DIR = "_aspectweavertempdir";
		public static string WorkDir;
		public static string BaseAssemblyPath;
		public static string[] AspectAssemblyPaths;

		public MainClass()
		{
		}

		public static void PrintUsage()
		{
			Console.Write("Usage:\nAspectWeaver.exe Path_to_AspectWeaverConfiguration.xml\n\n");
		}

		public static void CleanTempFiles()
		{
			if (System.IO.Directory.Exists(WorkDir + TEMP_DIR))
			{
				System.IO.Directory.Delete(WorkDir + TEMP_DIR, true);
			}
		}

		[STAThread]
		public static void Main(string[] args)
		{
			Console.Write("Teddy's Aspect Weaver Version 0.6\n\n");

			if (args != null && args.Length > 0)
			{
				if (System.IO.Path.GetDirectoryName(args[0]) == "")
				{
					WorkDir = ".\\";
				}
				else
				{
					WorkDir = System.IO.Path.GetDirectoryName(args[0]).TrimEnd('\\') + "\\";
				}
				XmlDocument doc = new ConfigurationVerifier(args[0]).DoVerify();

				if (doc != null)
				{
					BaseAssemblyPath = WorkDir + doc.SelectSingleNode("//BaseAssembly").InnerXml;
					XmlNodeList aspectAssemblyNodes = doc.SelectNodes("//AspectAssembly");
					if (aspectAssemblyNodes.Count > 0)
					{
						AspectAssemblyPaths = new string[aspectAssemblyNodes.Count];
						int i = 0;
						foreach (XmlNode node in aspectAssemblyNodes)
						{
							AspectAssemblyPaths[i++] = WorkDir + node.Attributes["path"].InnerXml;
						}
					}

					if (new Weaver(doc).DoWeave())
					{
						XmlAttribute cleanTempFilesAttr = doc.SelectSingleNode("/Configuration").Attributes["cleanTempFiles"];
						if (cleanTempFilesAttr != null && cleanTempFilesAttr.InnerXml == "true")
						{
							CleanTempFiles();
						}
					}
					else
					{
						Console.Write("Weaving error!\n\n");
					}
				}
				else
				{
					Console.Write("Configuration format error!\n\n");
				}
			}
			else
			{
				Console.Write("Incorrect arguments!\n\n");
				PrintUsage();
			}

#if DEBUG
			Console.Write("Press any key to continue...");
			Console.ReadLine();
#endif
		}
	}
}
