using System;
using System.Reflection;
using System.Xml;

namespace AspectWeaver
{
	public sealed class Utils
	{
		private Utils()
		{
		}

		public static Type DeepGetType(string type)
		{
			Type t = Type.GetType(type);
			if (t == null)
			{
				Assembly ass = Assembly.LoadFile(System.IO.Path.GetFullPath(AspectWeaver.MainClass.BaseAssemblyPath));
				t = ass.GetType(type, true);

				if (t == null)
				{
					foreach (string path in AspectWeaver.MainClass.AspectAssemblyPaths)
					{
						ass = Assembly.LoadFile(System.IO.Path.GetFullPath(path));
						t = ass.GetType(type, true);
						if (t != null)
						{
							break;
						}
					}
				}
			}
			if (t != null)
			{
				return t;
			}
			else
			{
				throw new Exception("Argument type \"" + type + "\" unloadable!\n\n");
			}
		}

		public static XmlNode CreateInstructionElement(XmlDocument baseDoc, int offset, string code, string operand, string guid)
		{
			XmlNode retNode = baseDoc.CreateElement("Instruction");
			retNode.Attributes.Append(CreateAttribute(baseDoc, "offset", offset.ToString()));
			XmlAttribute codeAttr = baseDoc.CreateAttribute("code");
			codeAttr.InnerXml = code;
			retNode.Attributes.Append(CreateAttribute(baseDoc, "code", code));
			if (operand != null)
			{
				retNode.Attributes.Append(CreateAttribute(baseDoc, "operand", operand));
			}
			if (guid != null)
			{
				retNode.Attributes.Append(CreateAttribute(baseDoc, "guid", guid));
			}
			return retNode;
		}

		public static XmlAttribute CreateAttribute(XmlDocument targetDoc, string attrName, string attrVal)
		{
			XmlAttribute newAttr = targetDoc.CreateAttribute(attrName);
			newAttr.InnerXml = attrVal;
			return newAttr;
		}

		public static void ReorderCodeNodeOffsets(XmlNodeList codeNodes)
		{
			if (codeNodes == null)
			{
				return;
			}

			int i = 0;
			foreach (XmlNode node in codeNodes)
			{
				if (node.LocalName == "Instruction")
				{
					node.Attributes["offset"].InnerXml = (i++).ToString();
				}
			}

			// reset bXXX Instruction's operand with new offset
			foreach (XmlNode codeNode in codeNodes)
			{
				if (codeNode.LocalName == "Instruction")
				{
					if ((codeNode.Attributes["code"].InnerXml.StartsWith("b") && codeNode.Attributes["code"].InnerXml != "box" && codeNode.Attributes["code"].InnerXml != "break") || codeNode.Attributes["code"].InnerXml.StartsWith("leave"))
					{
						XmlNode brNode = codeNode;
						XmlNode startNode = codeNode.ParentNode.SelectSingleNode("Instruction");
						string guid = (brNode.Attributes["guid"] == null ? null : brNode.Attributes["guid"].InnerXml);
						int offsetOfGuid = 0;
						bool isFindOffsetOdGuid = false;

						XmlNodeList tmpNodes = startNode.ParentNode.SelectNodes("Instruction[@guid = '" + (codeNode.Attributes["guid"] == null ? "" : codeNode.Attributes["guid"].InnerXml) + "']");
						string[] tmpXmls = new string[tmpNodes.Count];
						for (int j = 0; j < tmpNodes.Count; j++)
						{
							tmpXmls[j] = tmpNodes[j].OuterXml;
						}

						while (startNode != null && startNode.LocalName == "Instruction")
						{
							if ((startNode.Attributes["guid"] == null ? null : startNode.Attributes["guid"].InnerXml) == guid)
							{
								if (brNode.Attributes["operand"].InnerXml == offsetOfGuid.ToString())
								{
									brNode.Attributes["operand"].InnerXml = startNode.Attributes["offset"].InnerXml;
									isFindOffsetOdGuid = true;
									break;
								}

								offsetOfGuid++;
							}

							startNode = startNode.NextSibling;
						}

						if (!isFindOffsetOdGuid)
						{
							XmlNodeList fitGuidNodes = brNode.ParentNode.SelectNodes("Instruction[@guid='" + brNode.Attributes["guid"].InnerXml + "']");
							brNode.Attributes["operand"].InnerXml = fitGuidNodes[fitGuidNodes.Count - 1].NextSibling.Attributes["offset"].InnerXml;
						}
					}
				}
			}

			//reset code nodes guid
			string newGuid = Guid.NewGuid().ToString();
			foreach (XmlNode node in codeNodes)
			{
				if (node.LocalName == "Instruction")
				{
					if (node.Attributes["guid"] != null)
					{
						node.Attributes["guid"].InnerXml = newGuid;
					}
					else
					{
						node.Attributes.Append(Utils.CreateAttribute(codeNodes[0].OwnerDocument, "guid", newGuid));
					}
				}
			}

		}

		public static XmlNode CreateCloneNode(XmlDocument targetDoc, XmlNode sourceNode)
		{
			XmlNode cloneNode = targetDoc.CreateElement(sourceNode.LocalName);
			if (sourceNode.InnerXml != null && sourceNode.InnerXml.Length > 0)
			{
				cloneNode.InnerXml = sourceNode.InnerXml;
			}
			foreach (XmlAttribute attr in sourceNode.Attributes)
			{
				XmlAttribute newAttr = targetDoc.CreateAttribute(attr.LocalName);
				newAttr.InnerXml = attr.InnerXml;
				cloneNode.Attributes.Append(newAttr);
			}
			return cloneNode;
		}

		public static XmlNodeList CreateCloneCodeNodes(XmlNodeList sourceCodeNodes)
		{
			XmlDocument codeDoc = new XmlDocument();
			XmlNode rootNode = codeDoc.CreateElement("root");
			codeDoc.AppendChild(rootNode);

			// clone source local variable nodes
			foreach (XmlNode localNode in sourceCodeNodes[0].ParentNode.SelectNodes("Local"))
			{
				rootNode.AppendChild(Utils.CreateCloneNode(codeDoc, localNode));
			}

			// clone source instruction nodes
			string guid = Guid.NewGuid().ToString();
			foreach (XmlNode codeNode in sourceCodeNodes)
			{
				if (codeNode.LocalName == "Instruction")
				{
					XmlNode newCodeNode = CreateCloneNode(codeDoc, codeNode);
					if (codeNode.Attributes["guid"] == null)
					{
						newCodeNode.Attributes.Append(CreateAttribute(codeDoc, "guid", guid));
					}
					else
					{
						newCodeNode.Attributes["guid"].InnerXml = guid;
					}
					rootNode.AppendChild(newCodeNode);
				}
			}

			return rootNode.ChildNodes;
		}

		public static void ReplaceGetContextInfoWithLocalCode(XmlNodeList codeNodes, XmlNode targetMethodNode)
		{
			XmlNode rootNode = codeNodes[0].ParentNode;
			
			foreach (XmlNode getContextInfoNode in rootNode.SelectNodes("Instruction[@operand='System.String AspectWeaver.Aspects.Aspect::GetContextInfo()']"))
			{
				getContextInfoNode.Attributes["code"].InnerXml = "ldstr";
				getContextInfoNode.Attributes["operand"].InnerXml = targetMethodNode.Attributes["signature"].InnerXml;
			}
		}

		public static void ReplaceGetArgumentsWithLocalCode(XmlNodeList codeNodes, XmlNode targetMethodNode)
		{
			XmlNode rootNode = codeNodes[0].ParentNode;

			foreach (XmlNode getArgNode in rootNode.SelectNodes("Instruction[@operand='System.Object[] AspectWeaver.Aspects.Aspect::GetArguments()']"))
			{
				XmlDocument codeDoc = codeNodes[0].OwnerDocument;

				// set object[] size node
				XmlNodeList argNodes = targetMethodNode.SelectNodes("Parameter");
				int argCount = argNodes.Count;
				XmlAttribute guidAttr = getArgNode.Attributes["guid"];
				if (argCount > 8)
				{
					rootNode.InsertBefore(Utils.CreateInstructionElement(rootNode.OwnerDocument, 0, "ldc.i4.s", argCount.ToString(), (guidAttr == null ? null : guidAttr.InnerXml)), getArgNode);
				}
				else
				{
					rootNode.InsertBefore(Utils.CreateInstructionElement(rootNode.OwnerDocument, 0, "ldc.i4." + argCount.ToString(), null, (guidAttr == null ? null : guidAttr.InnerXml)), getArgNode);
				}
				if (guidAttr != null)
				{
					getArgNode.Attributes["guid"].InnerXml = Guid.NewGuid().ToString();
				}
				else
				{
					getArgNode.Attributes.Append(Utils.CreateAttribute(rootNode.OwnerDocument, "guid", Guid.NewGuid().ToString()));
				}

				// set newarr node
				getArgNode.Attributes["code"].InnerXml = "newarr";
				getArgNode.Attributes["operand"].InnerXml = "System.Object";

				// get newarr local index
				int locIndex = 0;

				string stloc = getArgNode.NextSibling.Attributes["code"].InnerXml;
				if (stloc == "stloc.s")
				{
					locIndex = int.Parse(getArgNode.NextSibling.Attributes["operand"].InnerXml);
				}
				else
				{
					locIndex = int.Parse(getArgNode.NextSibling.Attributes["code"].InnerXml.Substring(getArgNode.NextSibling.Attributes["code"].InnerXml.IndexOf(".") + 1));
				}

				// get ref node
				XmlNode refNode = getArgNode.NextSibling.NextSibling;

				string guid = Guid.NewGuid().ToString();

				// add get arg nodes
				int argStartIndex = 1;
				if (targetMethodNode.Attributes["isStatic"].InnerXml == "True")
				{
					argStartIndex = 0;
				}
				for (int i = 0; i < argNodes.Count; i++, argStartIndex++)
				{
					XmlNode argNode = argNodes[i];
					if (locIndex > 3)
					{
						rootNode.InsertBefore(CreateInstructionElement(codeDoc, -1, "ldloc.s", locIndex.ToString(), guid), refNode);
					}
					else
					{
						rootNode.InsertBefore(CreateInstructionElement(codeDoc, -1, "ldloc." + locIndex.ToString(), null, guid), refNode);
					}
					if (i > 8)
					{
						rootNode.InsertBefore(CreateInstructionElement(codeDoc, -1, "ldc.i4.s", i.ToString(), guid), refNode);
					}
					else
					{
						rootNode.InsertBefore(CreateInstructionElement(codeDoc, -1, "ldc.i4." + i.ToString(), null, guid), refNode);
					}
					if (argStartIndex > 3)
					{
						rootNode.InsertBefore(CreateInstructionElement(codeDoc, -1, "ldarg.s", argStartIndex.ToString(), guid), refNode);
					}
					else
					{
						rootNode.InsertBefore(CreateInstructionElement(codeDoc, -1, "ldarg." + argStartIndex.ToString(), null, guid), refNode);
					}
					if (DeepGetType(argNode.Attributes["type"].InnerXml).IsValueType)
					{
						rootNode.InsertBefore(CreateInstructionElement(codeDoc, -1, "box", argNode.Attributes["type"].InnerXml, guid), refNode);
					}
					rootNode.InsertBefore(CreateInstructionElement(codeDoc, -1, "stelem.ref", null, guid), refNode);
				}
			}

			ReorderCodeNodeOffsets(codeNodes);
		}

		public static void IntroduceLocalVariables(XmlNodeList codeNodes, XmlNode targetMethodNode)
		{
			int newLocalIndex = targetMethodNode.SelectNodes("Local").Count;

			// get aspect local variable node count
			XmlNodeList aspectLocalNodes = codeNodes[0].ParentNode.SelectNodes("Local");
			
			for (int i = 0; i < aspectLocalNodes.Count; i++)
			{
				// add local node
				XmlNode newLocalNode = Utils.CreateCloneNode(targetMethodNode.OwnerDocument, aspectLocalNodes[i]);
				newLocalNode.Attributes["index"].InnerXml = (newLocalIndex + i).ToString();
				targetMethodNode.InsertBefore(newLocalNode, targetMethodNode.SelectSingleNode("Instruction"));

				// set new code nodes' local node index
				foreach (XmlNode stOrLdNode in codeNodes[0].ParentNode.SelectNodes("Instruction[@code = 'stloc." + i.ToString() + "' or @code = 'ldloc." + i.ToString() + "']"))
				{
					if (stOrLdNode.Attributes["code"].InnerXml.StartsWith("stloc."))
					{
						if (newLocalIndex + i > 3)
						{
							stOrLdNode.Attributes["code"].InnerXml = "introloc" + "stloc.s";
							XmlAttribute operandAttr = stOrLdNode.Attributes["operand"];
							if (operandAttr != null)
							{
								operandAttr.InnerXml = (newLocalIndex + i).ToString();
							}
							else
							{
								stOrLdNode.Attributes.Append(Utils.CreateAttribute(stOrLdNode.OwnerDocument, "operand", (newLocalIndex + i).ToString()));
							}
						}
						else
						{
							stOrLdNode.Attributes["code"].InnerXml = "introloc" + "stloc." + (newLocalIndex + i).ToString();
						}
					}
					else
					{
						if (newLocalIndex + i > 3)
						{
							stOrLdNode.Attributes["code"].InnerXml = "introloc" + "ldloc.s";
							XmlAttribute operandAttr = stOrLdNode.Attributes["operand"];
							if (operandAttr != null)
							{
								operandAttr.InnerXml = (newLocalIndex + i).ToString();
							}
							else
							{
								stOrLdNode.Attributes.Append(Utils.CreateAttribute(stOrLdNode.OwnerDocument, "operand", (newLocalIndex + i).ToString()));
							}
						}
						else
						{
							stOrLdNode.Attributes["code"].InnerXml = "introloc" + "ldloc." + (newLocalIndex + i).ToString();
						}
					}
				}
			}

			foreach (XmlNode node in codeNodes)
			{
				if (node.LocalName == "Instruction")
				{
					if (node.Attributes["code"].InnerXml.StartsWith("introloc"))
					{
						node.Attributes["code"].InnerXml = node.Attributes["code"].InnerXml.Replace("introloc", "");
					}
				}
			}
		}

		public static string GetElementDesc(XmlNode node)
		{
			switch (node.LocalName)
			{
				case "Type":
					return node.Attributes["fullName"].InnerXml;
				case "Field":
					return node.Attributes["definition"].InnerXml;
				case "Property":
					return node.Attributes["definition"].InnerXml;
				case "Constructor":
					return node.Attributes["signature"].InnerXml;
				case "Method":
					return node.Attributes["signature"].InnerXml;
				default:
					return node.LocalName;
			}
		}

		public static XmlNode GetRefNodeBeforeConstructorOrMethodCall(XmlNode baseNode, int needArgCount)
		{
			int argCount = needArgCount;
			XmlNode refNode = baseNode;
			while (argCount > 0 && refNode.LocalName == "Instruction")
			{
				if (refNode.Attributes["code"].InnerXml.StartsWith("ld"))
				{
					argCount--;
				}
				else if (refNode.Attributes["code"].InnerXml == "call")
				{
					string returnAndType = refNode.Attributes["operand"].InnerXml.Substring(0, refNode.Attributes["operand"].InnerXml.IndexOf("::"));
					if (returnAndType.StartsWith("System.Void "))
					{
						argCount++;
					}
					string type = returnAndType.Substring(returnAndType.IndexOf(" ") + 1);
					string paramStr = refNode.Attributes["operand"].InnerXml.Substring(refNode.Attributes["operand"].InnerXml.IndexOf("("));
					if (paramStr != "()")
					{
						argCount += paramStr.Split(',').Length;
					}
					paramStr = paramStr.Trim('(', ')');
					string[] parms = paramStr.Split(',');
					if (parms != null && parms[0] == "")
					{
						parms = null;
					}
					if (parms == null)
					{
						parms = new string[0];
					}
					Type[] types = new Type[parms.Length];
					for (int i = 0; i < parms.Length; i++)
					{
						types[i] = DeepGetType(parms[i]);
					}
					string methodName = refNode.Attributes["operand"].InnerXml.Substring(returnAndType.Length + 2, refNode.Attributes["operand"].InnerXml.Length - 4 - paramStr.Length - returnAndType.Length);
					if (DeepGetType(type).GetMethod(methodName, types).IsStatic)
					{
						argCount--;
					}
				}
				else if (refNode.Attributes["code"].InnerXml == "pop")
				{
					argCount++;
				}
				else if (refNode.Attributes["code"].InnerXml == "newobj")
				{
					argCount--;
					string paramStr = refNode.Attributes["operand"].InnerXml.Substring(refNode.Attributes["operand"].InnerXml.IndexOf("("));
					if (paramStr != "()")
					{
						argCount += paramStr.Split(',').Length;
					}
				}
				else if (refNode.Attributes["code"].InnerXml.StartsWith("st"))
				{
					argCount++;
				}
										
				refNode = refNode.PreviousSibling;
			}
			return refNode.NextSibling;
		}

		public static void RemoveNodesFromTheirParentNode(XmlNodeList nodesToBeDeleted)
		{
			foreach (XmlNode nodeToBeDeleted in nodesToBeDeleted)
			{
				nodeToBeDeleted.ParentNode.RemoveChild(nodeToBeDeleted);
			}		
		}
		public static void InsertCodeNodesBefore(XmlNodeList codeNodes, XmlNode refNode)
		{
			string guid = Guid.NewGuid().ToString();

			foreach (XmlNode codeNode in codeNodes)
			{
				if (codeNode.LocalName == "Instruction")
				{
					if (refNode.ParentNode.SelectSingleNode("Instruction[@code = 'ret']") != null && codeNode.Attributes["code"].InnerXml == "ret")
					{
						continue;
					}

					XmlNode newCodeNode = Utils.CreateCloneNode(refNode.OwnerDocument, codeNode);
					if (codeNode.Attributes["guid"] == null)
					{
						newCodeNode.Attributes.Append(CreateAttribute(refNode.OwnerDocument, "guid", guid));
					}
					else
					{
						newCodeNode.Attributes["guid"].InnerXml = guid;
					}
					refNode.ParentNode.InsertBefore(newCodeNode, refNode);
				}
			}
							
			ReorderCodeNodeOffsets(refNode.ParentNode.ChildNodes);	
		}
	}
}
