using System;
using System.Xml;

namespace AspectWeaver
{
	public class WeaveLogger
	{
		private XmlDocument doc = new XmlDocument();
		private XmlNode rootNode;

		public WeaveLogger()
		{
			rootNode = doc.CreateElement("WeaveLog");
			doc.AppendChild(rootNode);
		}

		public void Save(string logFile)
		{
			doc.Save(logFile);
		}

		public XmlNode CreateLogNode(XmlNode adviceNode)
		{
			XmlNode logNode = doc.CreateElement("Weave");
			rootNode.AppendChild(logNode);

			XmlAttribute adviceAttr = doc.CreateAttribute("advice");
			adviceAttr.InnerXml = adviceNode.LocalName;
			logNode.Attributes.Append(adviceAttr);

			XmlAttribute statusAttr = doc.CreateAttribute("status");
			logNode.Attributes.Append(statusAttr);

			XmlAttribute targetXPathAttr = doc.CreateAttribute("targetXPath");
			targetXPathAttr.InnerXml = adviceNode.SelectSingleNode("Pointcut").Attributes["targetXPath"].InnerXml;
			logNode.Attributes.Append(targetXPathAttr);

			XmlNode codeNode = adviceNode.SelectSingleNode("Code");
			if (codeNode != null)
			{
				XmlAttribute aspectAssemblyAttr = doc.CreateAttribute("aspectAssembly");
				aspectAssemblyAttr.InnerXml = codeNode.Attributes["aspectAssemblyUniqueName"].InnerXml;
				logNode.Attributes.Append(aspectAssemblyAttr);
				XmlAttribute aspectXPathAttr = doc.CreateAttribute("aspectXPath");
				aspectXPathAttr.InnerXml = codeNode.Attributes["aspectXPath"].InnerXml;
				logNode.Attributes.Append(aspectXPathAttr);
			}

			return logNode;
		}

		public XmlNode CreateAffectedElementsNode()
		{
			return doc.CreateElement("AffectedElements");
		}
		public XmlNode CreateAffectedElementNode(string desc)
		{
			XmlNode affectedElement = doc.CreateElement("AffectedElement");
			affectedElement.InnerXml = desc;
			return affectedElement;
		}

	}
}
