using System;
using System.Reflection;
using System.Collections;
using System.Xml;
using AspectWeaver.Advices;

namespace AspectWeaver
{
	/// <summary>
	/// Summary description for Weaver.
	/// </summary>
	public class Weaver
	{
		private XmlDocument doc;
		private XmlDocument baseIlmlDoc = new XmlDocument();
		private Hashtable aspectIlmlDocs = new Hashtable();
		private WeaveLogger logger = new WeaveLogger();

		public Weaver(XmlDocument doc)
		{
			this.doc = doc;
		}

		public bool DoWeave()
		{
			try
			{
				XmlNode outputAssemblyNode = doc.SelectSingleNode("//OutputAssembly");
				if (!(outputAssemblyNode != null && outputAssemblyNode.InnerXml.Length > 0))
				{
					throw new Exception("/Configuration/OutputAssembly not exists or error!");
				}

				Ilml.IIlmlParser parser = Ilml.IlmlParserFactory.GetParser();

				// load base assembly ilml doc
				XmlNode baseAssemblyNode = doc.SelectSingleNode("//BaseAssembly");
				if (baseAssemblyNode != null && baseAssemblyNode.InnerXml.Length > 0)
				{
					string outputFilePath = AspectWeaver.MainClass.WorkDir + MainClass.TEMP_DIR + "\\BaseAssembly.ilml.xml";
					parser.Disassemble(new System.IO.FileInfo(AspectWeaver.MainClass.WorkDir + baseAssemblyNode.InnerXml), new System.IO.FileInfo(outputFilePath));
					baseIlmlDoc.Load(outputFilePath);
				}
				else
				{
					throw new Exception("/Configuration/BaseAssembly not exists or error!");
				}

				// load aspect assembly ilml docs
				foreach (XmlNode aspectAsemblyNode in doc.SelectNodes("//AspectAssembly"))
				{
					XmlDocument aspectAssemblyDoc = new XmlDocument();
					if (aspectAsemblyNode.Attributes["path"].InnerXml.Length > 0)
					{
						string outputFilePath = AspectWeaver.MainClass.WorkDir + MainClass.TEMP_DIR + "\\" + aspectAsemblyNode.Attributes["uniqueName"].InnerXml + ".ilml.xml";
						aspectAssemblyDoc.Load(outputFilePath);
					}
					else
					{
						throw new Exception("/Configuration/AspectAssembly not exists or error!");
					}
					aspectIlmlDocs.Add(aspectAsemblyNode.Attributes["uniqueName"].InnerXml, aspectAssemblyDoc);
				}

				AdviceFactory adviceFactory = new AdviceFactory(baseIlmlDoc, aspectIlmlDocs, logger);

				//do weaving
				foreach (XmlNode adviceNode in doc.SelectNodes("//Advices/*"))
				{
					adviceFactory.CreateAdvice(adviceNode).DoWeave();
				}

				// clear temp guid attributes
				foreach (XmlNode nodeWithGuid in baseIlmlDoc.SelectNodes("//Instruction[@guid]"))
				{
					nodeWithGuid.Attributes.Remove(nodeWithGuid.Attributes["guid"]);
				}

				// save weaved assembly to disk
				string outputAssemblyIlmlFilePath = AspectWeaver.MainClass.WorkDir + MainClass.TEMP_DIR + "\\OutputAssembly.ilml.xml";
				baseIlmlDoc.Save(outputAssemblyIlmlFilePath);
				parser.Assemble(new System.IO.FileInfo(outputAssemblyIlmlFilePath), new System.IO.FileInfo(AspectWeaver.MainClass.WorkDir + outputAssemblyNode.InnerXml));

				//save weave log
				XmlAttribute logFileAttr = doc.SelectSingleNode("/Configuration").Attributes["logFile"];
				if (logFileAttr != null && logFileAttr.InnerXml.Length > 0)
				{
					logger.Save(AspectWeaver.MainClass.WorkDir + logFileAttr.InnerXml);
				}

				Console.Write("Weaved successfully!\n\n");
			}
			catch(Exception e)
			{
				Console.Write(e.ToString() + "\n\n");
				return false;
			}

			return true;
		}
	}
}
