/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Evain Jb (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml {

    using System;
    using System.IO;
    using System.Xml;

    /// <summary>
    /// Parser used to deal with ILML
    /// </summary>
    public interface IIlmlParser {

        /// <summary>
        /// Disassemble an assembly into an ilml file
        /// stored on the disk
        /// </summary>
        /// <param name="input">The input assembly</param>
        /// <param name="output">The output ILML file</param>
        /// <exception cref="T:Ilml.Exceptions.AssemblyLoadException">
        /// Thrown if the input assembly can not be loaded</exception>
        /// <exception cref="T:Ilml.Exceptions.AssemblyParseException">
        /// Thrown if an errors occurs during the parsing process</exception>
        void Disassemble(FileInfo input, FileInfo output);

        /// <summary>
        /// Disassemble an assembly in an in memory
        /// XML representation
        /// </summary>
        /// <param name="input">The input assembly</param>
        /// <returns>The in memory ILML file</returns>
        /// <exception cref="T:Ilml.Exceptions.AssemblyLoadException">
        /// Thrown if the input assembly can not be loaded</exception>
        /// <exception cref="T:Ilml.Exceptions.AssemblyParseException">
        /// Thrown if an errors occurs during the parsing process</exception>
        XmlReader Disassemble(FileInfo input);

        /// <summary>
        /// Assemble an ILML file into an assembly
        /// </summary>
        /// <param name="input">The ILML file</param>
        /// <param name="output">The output assembly</param>
        /// <exception cref="T:Ilml.Exceptions.AssemblyLoadException">
        /// Thrown if the input assembly can not be loaded</exception>
        /// <exception cref="T:Ilml.Exceptions.AssemblyParseException">
        /// Thrown if an errors occurs during the parsing process</exception>
        void Assemble(FileInfo input, FileInfo output);

        /// <summary>
        /// Assemble an in memory ILML file
        /// into an assembly stored on the disk
        /// </summary>
        /// <param name="ilasm">The ILML reader</param>
        /// <param name="output">The ouput assembly</param>
        /// <exception cref="T:Ilml.Exceptions.AssemblyLoadException">
        /// Thrown if the input assembly can not be loaded</exception>
        /// <exception cref="T:Ilml.Exceptions.AssemblyParseException">
        /// Thrown if an errors occurs during the parsing process</exception>
        void Assemble(XmlReader ilasm, FileInfo output);

		/// <summary>
		/// Assemble an in memory ILML file
		/// into an assembly stored on the disk
		/// </summary>
		/// <param name="doc">The ILML XML Document</param>
		/// <param name="output">The ouput assembly</param>
		void Assemble(XmlDocument doc, FileInfo output);
    }
}