/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Thomas Gil (thomas.gil@dotnetguru.org)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml {

	using System;
	using System.Text;
	using System.Collections;
	using System.Diagnostics;
	using System.IO;
	using System.Xml;

    /// <summary>
    /// Classe utilitaire de log
    /// </summary>
	public class Logger {
		public static bool Debug;

		public static string LogIlmlPath;
		public static bool LogIlml;

		public static string LogWeavingPath;
		private static XmlDocument m_weaveDoc;
		private static XmlElement m_weaveDocRoot;
		private static bool m_logWeaving;
		public static bool LogWeaving {
			get { return m_logWeaving; }
			set {
				if (value){
					m_weaveDoc = new XmlDocument();
					m_weaveDocRoot = m_weaveDoc.CreateElement("WeaveLog");
					m_weaveDoc.AppendChild(m_weaveDocRoot);
				}
				m_logWeaving = value; 
			}
		}


		public static string LogWarningsPath;
		private static bool m_logWarnings;
		private static StringBuilder m_warningsBuffer;
		public static bool LogWarnings {
			get { return m_logWarnings; }
			set {
				if (value){
					if (m_warningsBuffer == null){
						m_warningsBuffer = new StringBuilder();
					}
				}
				m_logWarnings = value; 
			}
		}


		private static TextWriter m_out;
		static Logger() {
			m_out = System.Console.Out;
		}

        /// <summary>
        /// Log une information
        /// </summary>
        /// <param name="str">Une information</param>
		public static void Log(string str) {
			if (Debug) {
				m_out.WriteLine(str);	
			}
		}

		public static void LogWarning(string str) {
			if (LogWarnings) {
				m_warningsBuffer.Append(str);
			}
		}

        /// <summary>
        /// Log un string avec sa mise en forme
        /// </summary>
        /// <param name="format">Le pattern</param>
        /// <param name="args">Les arguments</param>
		public static void Log(string format, params object[] args) {
			if (Debug) {
				m_out.WriteLine(format, args);	
			}
		}

        /// <summary>
        /// Log une exception
        /// </summary>
        /// <param name="e">Une exception</param>
		public static void Log(Exception e) {
			Log(e.Message);
			Log(e.StackTrace);
		}

		private static IDictionary m_weaveElementsCache = new Hashtable();

		public static void Log(XmlElement ruleOrigin, string[] targets, string[] aspects){
			if (LogWeaving) {

				XmlElement weave = m_weaveElementsCache[ruleOrigin] as XmlElement;
				if (weave == null){
					m_weaveElementsCache[ruleOrigin] = weave = m_weaveDoc.CreateElement("Weave");				
					weave.SetAttribute("name", ruleOrigin.Name);
					m_weaveDocRoot.AppendChild(weave);

					foreach(XmlAttribute att in ruleOrigin.Attributes){
						weave.SetAttribute(att.Name, att.Value);
					}
				}

				XmlElement weaveStep = m_weaveDoc.CreateElement("WeaveStep");
				weave.AppendChild(weaveStep);

				foreach(string aspect in aspects){
					if (aspect != string.Empty){
						XmlElement aspectInfo = m_weaveDoc.CreateElement("AspectInfo");
						aspectInfo.InnerText = aspect;
						weaveStep.AppendChild(aspectInfo);
					}
				}

				foreach(string target in targets){
					XmlElement targetInfo = m_weaveDoc.CreateElement("TargetInfo");
					targetInfo.InnerText = target;
					weaveStep.AppendChild(targetInfo);
				}
			}
		}

		public static void Log(XmlElement ruleOrigin, string target, string aspect){
			if (LogWeaving) {
				Log(ruleOrigin, new string[]{target}, new string[]{aspect});
			}
		}

		public static void SaveIlml(XmlDocument ilml) {
			if (LogIlml) {			
				ilml.Save(LogIlmlPath);
			}			
		}

		public static void SaveWeaving() {
			if (LogWeaving) {			
				m_weaveDoc.Save(LogWeavingPath);
			}			
		}

		public static void SaveWarnings() {
			if (LogWarnings) {			
				StreamWriter sout = new StreamWriter(LogWarningsPath);
				sout.WriteLine(m_warningsBuffer);
				sout.Close();
			}			
		}
	}	
}