/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Evain Jb (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml {

	using System;
	using System.Reflection;
	using System.IO;
    using System.Xml;
    using System.Xml.XPath;

    using Rail.Reflect;

    using Ilml.Adapters;

	internal class RailParser : IIlmlParser {
        public void Disassemble(FileInfo input, FileInfo output) {
            RAssemblyDef asm = null;
            try {
                asm = RAssemblyDef.LoadAssembly(input.FullName);
            } catch (Exception e) {
                throw new AssemblyLoadException(
                    "Error while loading assembly", e);
            }
            try {
                XmlTextWriter writer = new XmlTextWriter(output.FullName, System.Text.Encoding.UTF8);
				writer.Formatting = Formatting.Indented;
                AssemblyAdapter.Instance.ToXml(asm, writer);
                writer.Flush();
                writer.Close();
            } catch (DisassembleException) {
                throw;
            } catch (Exception e) {
                throw new DisassembleException(
                    "Error while disassembling assembly", e);
            }
        }

        public XmlReader Disassemble(FileInfo input) {
            StringWriter sw = new StringWriter();
            XmlWriter writer = new XmlTextWriter(sw);
            RAssemblyDef asm = null;
            XmlTextReader ret = null;
            try {
                asm = RAssemblyDef.LoadAssembly(input.FullName);
            } catch (Exception e) {
                throw new AssemblyLoadException(
                    "Error while loading assembly", e);
            }
            try {
                AssemblyAdapter.Instance.ToXml(asm, writer);
                ret = new XmlTextReader(
                    sw.GetStringBuilder().ToString(),
                   XmlNodeType.Document, null);
            } catch (DisassembleException) {
                throw;
            } catch (Exception e) {
                throw new DisassembleException(
                    "Error while disassembling assembly", e);
            }
            return ret;
        }

        public void Assemble(FileInfo input, FileInfo output) {
            XmlDocument doc = null;
            try {
                doc = new XmlDocument();
                doc.Load(input.FullName);
            } catch (Exception e) {
                throw new AssemblyLoadException(
                    "Error while loading assembly", e);
            }
            try {
                Assemble(doc, output);
            } catch (AssembleException) {
                throw;
            } catch (Exception e) {
                throw new AssembleException(
                    "Error while assembling assembly", e);
            }
        }

        public void Assemble(XmlReader ilasm, FileInfo output) {
            XmlDocument doc = null;
            try {
                doc = new XmlDocument();
                doc.Load(ilasm);
            } catch (Exception e) {
                throw new AssemblyLoadException(
                    "Error while loading assembly", e);
            }
            try {
                Assemble(doc, output);
            } catch (AssembleException) {
                throw;
            } catch (Exception e) {
                throw new AssembleException(
                    "Error while assembling assembly", e);
            }
        }

        public void Assemble(XmlDocument doc, FileInfo output) {
            XPathNavigator nav = doc.CreateNavigator();
            nav.MoveToFirstChild();
			RAssemblyDef asm = AssemblyAdapter.Instance.FromXml(nav);
			
			ClearPreviousResults(output);

			asm.SaveAssembly(output.Name);

			MoveResults(output);
		}

        private FileInfo GetPdb(FileInfo asm) {
            string pdb = asm.FullName.Substring(0,
                asm.FullName.LastIndexOf("."));
            return new FileInfo(string.Concat(pdb, ".pdb"));
        }

        private void ClearPreviousResults(FileInfo output) {
            if (File.Exists(output.FullName)) {
                File.Delete(output.FullName);
            }
            FileInfo pdb = GetPdb(output);
            if (File.Exists(pdb.FullName)) {
                File.Delete(pdb.FullName);
            }
        }

        private void MoveResults(FileInfo output) {
            FileInfo outpdb = GetPdb(output);

            FileInfo[] files = new FileInfo[] { output, outpdb };

            foreach (FileInfo fi in files) {
                if (File.Exists(fi.Name) && !File.Exists(fi.FullName)) {
                    File.Move(fi.Name, fi.FullName);
                }
            }
        }
	}
}
