/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Evain Jb   (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

    using System;
    using System.Collections;
    using System.Globalization;
    using System.Xml;
    using System.Xml.XPath;

    using Rail.Reflect;

    internal sealed class AssemblyNameAdapter : XmlAdapter {

        #region singleton implementation

        private AssemblyNameAdapter() : base("AssemblyName")  {}
        public static readonly AssemblyNameAdapter Instance = 
            new AssemblyNameAdapter();

        #endregion

        public void ToXml(RAssemblyName name, XmlWriter writer) {
            writer.WriteStartElement(TagName, Utils.ilNS);

            writer.WriteAttributeString("name", Utils.ilNS,
                name.Name);

            if (name.CultureInfo.Name.Length > 0) {
                writer.WriteAttributeString("culture", Utils.ilNS,
                    name.CultureInfo.Name);
            }

            if (name.PublicKeyToken != null) {
                writer.WriteAttributeString("publicKeyToken", Utils.ilNS,
                    Convert.ToBase64String(name.PublicKeyToken));
            }

            VersionAdapter.Instance.ToXml(name.Version, writer);

            writer.WriteEndElement();
        }

        public RAssemblyNameDef FromXml(XPathNavigator nav) {
            RAssemblyNameDef name = new RAssemblyNameDef(nav.GetAttribute(
                "name", Utils.ilNS));

            if (Utils.IsAttributePresent(nav, "culture")) {
                string culture = nav.GetAttribute("culture", Utils.ilNS);
                foreach (CultureInfo ci in CultureInfo.GetCultures(
                    CultureTypes.AllCultures)) {

                    if (ci.Name == culture) {
                        name.CultureInfo = ci;
                    }
                }
            }
            
            if (Utils.IsAttributePresent(nav, "publicKeyToken")) {
                name.PublicKeyToken = Convert.FromBase64String(
                    nav.GetAttribute("publicKeyToken", Utils.ilNS));
            }

            XPathNodeIterator it = nav.Select(VersionAdapter.Instance.TagName);
            it.MoveNext();
            VersionAdapter.Instance.FromXml(name, it.Current);

            return name;
        }
    }
}
