/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Evain Jb   (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

    using System;
    using System.Collections;
    using System.Xml;
    using System.Xml.XPath;

    using Rail.Reflect;

	internal sealed class AttributeAdapter : XmlAdapter {

        #region singleton implementation

		private AttributeAdapter() : base("Attribute")  {}
		public static readonly AttributeAdapter Instance = 
			new AttributeAdapter();

        #endregion

		public void ToXml(RCustomAttrib attr, XmlWriter writer) {
			writer.WriteStartElement(TagName, Utils.ilNS);
			writer.WriteAttributeString("type", Utils.ilNS,
                Utils.GetFullyQualifiedName(attr.CustomAttributeType));

            writer.WriteAttributeString("constructorSignature", Utils.ilNS,
                Utils.GetSignature(attr.AttributeConstructor));
            
            writer.WriteStartAttribute("data", Utils.ilNS);
            writer.WriteBase64(attr.AttributeData, 0,
                attr.AttributeData.Length);
            writer.WriteEndAttribute();

			writer.WriteEndElement();
		}

        public void FromXml(ICustomAttributeOwnerDef owner,
            XPathNavigator nav) {

            byte[] data = Convert.FromBase64String(
                nav.GetAttribute("data", Utils.ilNS));
            
            RCustomAttrib attr = new RCustomAttrib(
                ConstructorAdapter.Instance.GetConstructor(
                    nav.GetAttribute("constructorSignature", Utils.ilNS)),
                data, AssemblyAdapter.Instance.CurrentAssembly);

            owner.AddCustomAttributtes(attr);
        }
	}
}
