/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Evain Jb   (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

	using System;
	using System.Collections;
	using System.Reflection.Emit;
	using System.Text;

	using Rail.MSIL;
	using Rail.Reflect;

	internal sealed class CILFactory {

		#region singleton implementation

		private CILFactory() {
			m_opCodes = new Hashtable();
		}

		private static CILFactory m_instance;

		public static CILFactory Instance {
			get {
				if (m_instance == null) {
					m_instance = new CILFactory();
				}
				return m_instance;
			}
		}

		#endregion

		private IDictionary m_opCodes;

		private OpCode GetOpCode(string name) {
			OpCode code;
			if (m_opCodes.Contains(name)) {
				code = (OpCode)m_opCodes[name];
			} else {
				string[] tokens = name.Split(new char[]{'.'});
				StringBuilder buffer = new StringBuilder(name.Length);
				for (int i = 0 ; i < tokens.Length ; i++) {
					if (tokens[i].Length > 0) {
						if (i > 0) {
							buffer.Append("_");
						}
						for (int j = 0 ; j < tokens[i].Length ; j++) {
							if (j == 0) {
								buffer.Append(
									tokens[i][j].ToString().ToUpper());
							} else {
								buffer.Append(tokens[i][j]);
							}
						}
					}
				}
				string field = buffer.ToString();

				m_opCodes[name] = code = (OpCode)typeof(OpCodes).GetField(
					field).GetValue(null);
			}
			return code;
		}

		public Instruction GetInstruction(string op) {
			#region DEBUG
#if DEBUG
			Logger.Log("GetInstruction({0})", op);
#endif 
			#endregion
			OpCode code = GetOpCode(op);
			return DefineInstruction(code);
		}


		private Instruction DefineInstruction(OpCode code) {
			switch (code.OperandType) {
				case OperandType.ShortInlineBrTarget :
				case OperandType.InlineBrTarget :
					return new ILBranch(code, null);
				case OperandType.InlineField :
					return new ILField(code, null);
				case OperandType.InlineMethod :
					return new ILMethod(code, null);
				case OperandType.InlineNone :
					return new ILNone(code);
				case OperandType.InlineString :
					return new ILString(code, null);
				case OperandType.InlineTok :
					return new ILToken(code, null);
				case OperandType.InlineSwitch :
					return new ILSwitch(code, null);
				case OperandType.InlineType :
					return new ILType(code, null);
				case OperandType.ShortInlineI :
					return new ILOperandI2(code, 0);
				case OperandType.InlineI :
					return new ILOperandI4(code, 0);
				case OperandType.InlineI8 :
					return new ILOperandI8(code, 0);
				case OperandType.ShortInlineR :
					return new ILOperandR4(code, 0);
				case OperandType.InlineR :
					return new ILOperandR8(code, 0);
				case OperandType.ShortInlineVar :
				case OperandType.InlineVar :
					string type = code.Name.Substring(2, 3);
				switch (type) {
					case "loc" :
						return new ILLocalVariable(code, null);
					case "arg" :
						return new ILArgument(code, null);
					default :
						return null;
				}
				default :
					throw new ApplicationException(
						"OperandType not supported");
			}
		}
	}
}