/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Thomas Gil (thomas.gil@dotnetguru.org)
 *   Evain Jb   (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

    using System;
    using System.Collections;
    using System.Reflection;
    using System.Xml;
    using System.Xml.XPath;

    using Rail.MSIL;
    using Rail.Reflect;

	internal sealed class ConstructorAdapter : XmlAdapter {

        #region singleton implementation

		private ConstructorAdapter() : base("Constructor") {
            m_constructors = new Hashtable();
        }
		public static readonly ConstructorAdapter Instance = 
			new ConstructorAdapter();

        #endregion

		private IDictionary m_constructors;

        public void ToXml(RConstructorDef c, XmlWriter writer) {
            writer.WriteStartElement(TagName, Utils.ilNS);	

            writer.WriteAttributeString("name", Utils.ilNS, c.Name);
            writer.WriteAttributeString("signature", Utils.ilNS,
                Utils.GetSignature(c));

            #region CallingConventions writing

            bool b;
            
            // Calling conventions attributes
            b = (c.CallingConventions & CallingConventions.Standard) ==
                CallingConventions.Standard;
            writer.WriteAttributeString("isStandard", Utils.ilNS,
                b.ToString());
            b = (c.CallingConventions & CallingConventions.VarArgs) ==
                CallingConventions.VarArgs;
            writer.WriteAttributeString("isVarArgs", Utils.ilNS,
                b.ToString());
            b = (c.CallingConventions & CallingConventions.Any) ==
                CallingConventions.Any;
            writer.WriteAttributeString("isAny", Utils.ilNS,
                b.ToString());
            b = (c.CallingConventions & CallingConventions.HasThis) ==
                CallingConventions.HasThis;
            writer.WriteAttributeString("isHasThis", Utils.ilNS,
                b.ToString());
            b = (c.CallingConventions & CallingConventions.ExplicitThis) ==
                CallingConventions.ExplicitThis;
            writer.WriteAttributeString("isExplicitThis", Utils.ilNS,
                b.ToString());

            #endregion

            #region MethodAttributes writing

            // Method access attributes
            int mask = (int)(c.Attributes & MethodAttributes.MemberAccessMask);
            b = mask == (int)MethodAttributes.PrivateScope;
            writer.WriteAttributeString("isPrivateScope", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodAttributes.Private;
            writer.WriteAttributeString("isPrivate", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodAttributes.FamANDAssem;
            writer.WriteAttributeString("isFamANDAssem", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodAttributes.Assembly;
            writer.WriteAttributeString("isAssembly", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodAttributes.Family;
            writer.WriteAttributeString("isFamily", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodAttributes.FamORAssem;
            writer.WriteAttributeString("isFamORAssem", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodAttributes.Public;
            writer.WriteAttributeString("isPublic", Utils.ilNS,
                b.ToString());

            // Method contract attributes
            b = (c.Attributes & MethodAttributes.Static) ==
                MethodAttributes.Static;
            writer.WriteAttributeString("isStatic", Utils.ilNS,
                b.ToString());
            b = (c.Attributes & MethodAttributes.Final) ==
                MethodAttributes.Final;
            writer.WriteAttributeString("isFinal", Utils.ilNS,
                b.ToString());
            b = (c.Attributes & MethodAttributes.Virtual) ==
                MethodAttributes.Virtual;
            writer.WriteAttributeString("isVirtual", Utils.ilNS,
                b.ToString());
            b = (c.Attributes & MethodAttributes.HideBySig) ==
                MethodAttributes.HideBySig;
            writer.WriteAttributeString("isHideBySig", Utils.ilNS,
                b.ToString());

            // Vtable layout attributes
            mask = (int)(c.Attributes & MethodAttributes.VtableLayoutMask);
            b = mask == (int)MethodAttributes.ReuseSlot;
            writer.WriteAttributeString("isReuseSlot", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodAttributes.NewSlot;
            writer.WriteAttributeString("isNewSlot", Utils.ilNS,
                b.ToString());

            // Method implementation attributes
            b = (c.Attributes & MethodAttributes.Abstract) ==
                MethodAttributes.Abstract;
            writer.WriteAttributeString("isAbstract", Utils.ilNS,
                b.ToString());
            b = (c.Attributes & MethodAttributes.SpecialName) ==
                MethodAttributes.SpecialName;
            writer.WriteAttributeString("isSpecialName", Utils.ilNS,
                b.ToString());

            // Interop attributes
            b = (c.Attributes & MethodAttributes.PinvokeImpl) ==
                MethodAttributes.PinvokeImpl;
            writer.WriteAttributeString("isPinvokeImpl", Utils.ilNS,
                b.ToString());
            b = (c.Attributes & MethodAttributes.UnmanagedExport) ==
                MethodAttributes.UnmanagedExport;
            writer.WriteAttributeString("isUnmanagedExport", Utils.ilNS,
                b.ToString());
            b = (c.Attributes & MethodAttributes.RTSpecialName) ==
                MethodAttributes.RTSpecialName;
            writer.WriteAttributeString("isRTSpecialName", Utils.ilNS,
                b.ToString());

            // Runtime reserved attributes
            mask = (int)(c.Attributes & MethodAttributes.ReservedMask);
            b = mask == (int)MethodAttributes.HasSecurity;
            writer.WriteAttributeString("isHasSecurity", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodAttributes.RequireSecObject;
            writer.WriteAttributeString("isRequireSecObject", Utils.ilNS,
                b.ToString());

            #endregion

            #region MethodImplementation writing

            // Code implementation attributes
            mask = (int)(c.MethodImplementationFlags &
                MethodImplAttributes.CodeTypeMask);
            b = mask == (int)MethodImplAttributes.IL;
            writer.WriteAttributeString("isIL", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodImplAttributes.Native;
            writer.WriteAttributeString("isNative", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodImplAttributes.OPTIL;
            writer.WriteAttributeString("isOPTIL", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodImplAttributes.Runtime;
            writer.WriteAttributeString("isRuntime", Utils.ilNS,
                b.ToString());

            // Managed implementation attributes
            mask = (int)(c.MethodImplementationFlags &
                MethodImplAttributes.ManagedMask);
            b = mask == (int)MethodImplAttributes.Unmanaged;
            writer.WriteAttributeString("isUnmanaged", Utils.ilNS,
                b.ToString());
            b = mask == (int)MethodImplAttributes.Managed;
            writer.WriteAttributeString("isManaged", Utils.ilNS,
                b.ToString());

            // Implementation and interop attributes
            b = (c.MethodImplementationFlags &
                MethodImplAttributes.ForwardRef) == 
                MethodImplAttributes.ForwardRef;
            writer.WriteAttributeString("isForwardRef", Utils.ilNS,
                b.ToString());
            b = (c.MethodImplementationFlags &
                MethodImplAttributes.PreserveSig) == 
                MethodImplAttributes.PreserveSig;
            writer.WriteAttributeString("isPreserveSig", Utils.ilNS,
                b.ToString());
            b = (c.MethodImplementationFlags &
                MethodImplAttributes.InternalCall) == 
                MethodImplAttributes.InternalCall;
            writer.WriteAttributeString("isInternalCall", Utils.ilNS,
                b.ToString());
            b = (c.MethodImplementationFlags &
                MethodImplAttributes.Synchronized) == 
                MethodImplAttributes.Synchronized;
            writer.WriteAttributeString("isSynchronized", Utils.ilNS,
                b.ToString());
            b = (c.MethodImplementationFlags &
                MethodImplAttributes.NoInlining) == 
                MethodImplAttributes.NoInlining;
            writer.WriteAttributeString("isNoInlining", Utils.ilNS,
                b.ToString());

            #endregion
			
            foreach (RCustomAttrib attr in c.GetCustomAttributes(false)) {
                AttributeAdapter.Instance.ToXml(attr, writer);
            }

            foreach (RParameter param in c.GetParameters()) {
                ParameterAdapter.Instance.ToXml(param, writer);
            }

            if (c.MethodBody != null) {
                for (int i = 0 ; i < c.MethodBody.LocalVarsCount ; i++) {
                    LocalAdapter.Instance.ToXml(c.MethodBody.GetLocalVar(i),
                        i, writer);
                }
                CodeTree t = new CodeTreeReflect(c.MethodBody);
                for (CodeNode node = t.Root ; node != null ; node = node.Next) {
                    if (node is CodeNodeInstruction) {
                        InstructionAdapter.Instance.ToXml(
                            (CodeNodeInstruction)node, writer);
                    } else if (node is CodeNodeException) {
                        ExceptionAdapter.Instance.ToXml(
                            (CodeNodeException)node, writer);
                    }
                }
            }

            writer.WriteEndElement();
        }

        public void FromXml(RTypeDef t, XPathNavigator nav) {
            if (m_constructors.Contains(Utils.GetSignature(nav))) {
                return;
            }

            RConstructorDef c = t.DefineConstructor(
                ParameterAdapter.Instance.FromXml(
                    nav.Select(ParameterAdapter.Instance.TagName)),
                nav.GetAttribute("name", Utils.ilNS));

            #region CallingConventions reading

            CallingConventions cc = new CallingConventions();
            
            // Calling conventions attributes
            if (Utils.IsAttributeTrue(nav, "isStandard")) {
                cc |= CallingConventions.Standard;
            }
            if (Utils.IsAttributeTrue(nav, "isVarArgs")) {
                cc |= CallingConventions.VarArgs;
            }
            if (Utils.IsAttributeTrue(nav, "isAny")) {
                cc |= CallingConventions.Any;
            }
            if (Utils.IsAttributeTrue(nav, "isHasThis")) {
                cc |= CallingConventions.HasThis;
            }
            if (Utils.IsAttributeTrue(nav, "isExplicitThis")) {
                cc |= CallingConventions.ExplicitThis;
            }

            c.CallingConventions = cc;

            #endregion

            #region MethodAttributes writing

            MethodAttributes attrs = new MethodAttributes();

            // Method access attributes
            if (Utils.IsAttributeTrue(nav, "isPrivateScope")) {
                attrs |= MethodAttributes.MemberAccessMask &
                    MethodAttributes.PrivateScope;
            }
            if (Utils.IsAttributeTrue(nav, "isPrivate")) {
                attrs |= MethodAttributes.MemberAccessMask &
                    MethodAttributes.Private;
            }
            if (Utils.IsAttributeTrue(nav, "isFamANDAssem")) {
                attrs |= MethodAttributes.MemberAccessMask &
                    MethodAttributes.FamANDAssem;
            }
            if (Utils.IsAttributeTrue(nav, "isAssembly")) {
                attrs |= MethodAttributes.MemberAccessMask &
                    MethodAttributes.Assembly;
            }
            if (Utils.IsAttributeTrue(nav, "isFamily")) {
                attrs |= MethodAttributes.MemberAccessMask &
                    MethodAttributes.Family;
            }
            if (Utils.IsAttributeTrue(nav, "isFamORAssem")) {
                attrs |= MethodAttributes.MemberAccessMask &
                    MethodAttributes.FamORAssem;
            }
            if (Utils.IsAttributeTrue(nav, "isPublic")) {
                attrs |= MethodAttributes.MemberAccessMask &
                    MethodAttributes.Public;
            }

            // Method contract attributes
            if (Utils.IsAttributeTrue(nav, "isStatic")) {
                attrs |= MethodAttributes.Static;
            }
            if (Utils.IsAttributeTrue(nav, "isFinal")) {
                attrs |= MethodAttributes.Final;
            }
            if (Utils.IsAttributeTrue(nav, "isVirtual")) {
                attrs |= MethodAttributes.Virtual;
            }
            if (Utils.IsAttributeTrue(nav, "isHideBySig")) {
                attrs |= MethodAttributes.HideBySig;
            }

            // Vtable layout attributes
            if (Utils.IsAttributeTrue(nav, "isReuseSlot")) {
                attrs |= MethodAttributes.VtableLayoutMask &
                    MethodAttributes.ReuseSlot;
            }
            if (Utils.IsAttributeTrue(nav, "isNewSlot")) {
                attrs |= MethodAttributes.VtableLayoutMask &
                    MethodAttributes.NewSlot;
            }

            // Method implementation attributes
            if (Utils.IsAttributeTrue(nav, "isAbstract")) {
                attrs |= MethodAttributes.Abstract;
            }
            if (Utils.IsAttributeTrue(nav, "isSpecialName")) {
                attrs |= MethodAttributes.SpecialName;
            }

            // Interop attributes
            if (Utils.IsAttributeTrue(nav, "isPinvokeImpl")) {
                attrs |= MethodAttributes.PinvokeImpl;
            }
            if (Utils.IsAttributeTrue(nav, "isUnmanagedExport")) {
                attrs |= MethodAttributes.UnmanagedExport;
            }
            if (Utils.IsAttributeTrue(nav, "isRTSpecialName")) {
                attrs |= MethodAttributes.RTSpecialName;
            }

            // Runtime reserved attributes
            if (Utils.IsAttributeTrue(nav, "isHasSecurity")) {
                attrs |= MethodAttributes.ReservedMask &
                    MethodAttributes.HasSecurity;
            }
            if (Utils.IsAttributeTrue(nav, "isRequireSecObject")) {
                attrs |= MethodAttributes.ReservedMask &
                    MethodAttributes.RequireSecObject;
            }

            c.Attributes = attrs;

            #endregion

            #region MethodImplementation writing

            MethodImplAttributes mia = new MethodImplAttributes();

            // Code implementation attributes
            if (Utils.IsAttributeTrue(nav, "isIL")) {
                mia |= MethodImplAttributes.CodeTypeMask &
                    MethodImplAttributes.IL;
            }
            if (Utils.IsAttributeTrue(nav, "isNative")) {
                mia |= MethodImplAttributes.CodeTypeMask &
                    MethodImplAttributes.Native;
            }
            if (Utils.IsAttributeTrue(nav, "isOPTIL")) {
                mia |= MethodImplAttributes.CodeTypeMask &
                    MethodImplAttributes.OPTIL;
            }
            if (Utils.IsAttributeTrue(nav, "isRuntime")) {
                mia |= MethodImplAttributes.CodeTypeMask &
                    MethodImplAttributes.Runtime;
            }

            // Managed implementation attributes
            if (Utils.IsAttributeTrue(nav, "isUnmanaged")) {
                mia |= MethodImplAttributes.ManagedMask &
                    MethodImplAttributes.Unmanaged;
            }
            if (Utils.IsAttributeTrue(nav, "isManaged")) {
                mia |= MethodImplAttributes.ManagedMask &
                    MethodImplAttributes.Managed;
            }

            // Implementation and interop attributes
            if (Utils.IsAttributeTrue(nav, "isForwardRef")) {
                mia |= MethodImplAttributes.ForwardRef;
            }
            if (Utils.IsAttributeTrue(nav, "isPreserveSig")) {
                mia |= MethodImplAttributes.PreserveSig;
            }
            if (Utils.IsAttributeTrue(nav, "isInternalCall")) {
                mia |= MethodImplAttributes.InternalCall;
            }
            if (Utils.IsAttributeTrue(nav, "isSynchronized")) {
                mia |= MethodImplAttributes.Synchronized;
            }
            if (Utils.IsAttributeTrue(nav, "isNoInlining")) {
                mia |= MethodImplAttributes.NoInlining;
            }

            c.MethodImplementationFlags = mia;

            #endregion

            m_constructors.Add(Utils.GetSignature(nav), c);

            XPathNodeIterator it = nav.Select(
                AttributeAdapter.Instance.TagName);
            while (it.MoveNext()) {
                AttributeAdapter.Instance.FromXml(c, it.Current);
            }

            int ins = nav.Select(InstructionAdapter.Instance.TagName).Count;
            if (ins > 0) {
                CodeTree tree = new CodeTreeXml(nav);
                MethodBody mb = c.DefineMethodBody();
                it = nav.Select(LocalAdapter.Instance.TagName);
                while (it.MoveNext()) {
                    LocalAdapter.Instance.FromXml(mb, it.Current);
                }
                CodeNode node = tree.Root;
                while (node != null) {
                    if (node is CodeNodeInstruction) {
                        InstructionAdapter.Instance.FromXml(mb,
                            (CodeNodeInstruction)node);
                    } else if (node is CodeNodeException) {
                        ExceptionAdapter.Instance.FromXml(mb,
                            (CodeNodeException)node);
                    }
                    node = node.Next;
                }
            }
        }
        
        public RConstructor GetConstructor(string signature) {
			#region DEBUG
#if DEBUG
			Logger.Log("GetConstructor({0})", signature);
#endif 
			#endregion
			RConstructor result = 
                (RConstructor)m_constructors[signature];
            if (result == null) {
                RType t = TypeAdapter.Instance.GetType(
                    Utils.GetMemberTypeInSigOrDef(signature));
                foreach (RConstructor c in t.GetConstructors()) {
                    if (Utils.GetSignature(c) == signature) {
                        result = c;
                        break;
                    }
                }
                if (result != null) {
                    m_constructors[signature] = result;
                }
            }
            if (result == null) {
                XPathNodeIterator it =
                    AssemblyAdapter.Instance.Navigator.Select(string.Format(
                    "//Constructor[@signature='{0}']", signature));
                while (it.MoveNext()) {
                    RTypeDef type = (RTypeDef)TypeAdapter.Instance.GetType(
                        Utils.GetParentFullName(it.Current));
                    FromXml(type, it.Current);
                    result = (RConstructor)m_constructors[signature];
                }
            }
            return result;
        }
	}
}
