/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Evain Jb   (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

    using System;
    using System.Reflection;
    using System.Xml;
    using System.Xml.XPath;

    using Rail.Reflect;

    internal sealed class EventAdapter : XmlAdapter {

        #region singleton implementation

        private EventAdapter() : base("Event")  {}
        public static readonly EventAdapter Instance = 
            new EventAdapter();

        #endregion

        public void ToXml(REventDef e, XmlWriter writer) {
            writer.WriteStartElement(TagName, Utils.ilNS);

            writer.WriteAttributeString("name", Utils.ilNS, e.Name);

            writer.WriteAttributeString("type", Utils.ilNS,
                Utils.GetFullyQualifiedName(e.EventType));

            writer.WriteAttributeString("addOnMethodSignature", Utils.ilNS,
                Utils.GetSignature(e.AddOnMethod));
            writer.WriteAttributeString("removeOnMethodSignature", Utils.ilNS,
                Utils.GetSignature(e.RemoveOnMethod));
            if (e.RaiseMethod != null) {
                writer.WriteAttributeString("raiseMethodSignature", Utils.ilNS,
                    Utils.GetSignature(e.RaiseMethod));
            }

            #region EventAttributes writing

            bool b;

            // Generals attributes
            b = (e.Attributes & EventAttributes.SpecialName) ==
                EventAttributes.SpecialName;
            writer.WriteAttributeString("isSpecialName", Utils.ilNS,
                b.ToString());
            b = e.Attributes == EventAttributes.None;
            writer.WriteAttributeString("isNone", Utils.ilNS,
                b.ToString());

            // Runtime reserved attributes
            int mask = (int)(e.Attributes & EventAttributes.ReservedMask);
            b = mask == (int)EventAttributes.RTSpecialName;
            writer.WriteAttributeString("isRTSpecialName", Utils.ilNS,
                b.ToString());

            #endregion

            writer.WriteEndElement();
        }

        public void FromXml(RTypeDef t, XPathNavigator nav) {
            REventDef e = t.DefineEvent(
                nav.GetAttribute("name", Utils.ilNS),
                TypeAdapter.Instance.GetType(
                    nav.GetAttribute("type", Utils.ilNS)));

            e.AddOnMethod = MethodAdapter.Instance.GetMethod(
                nav.GetAttribute("addOnMethodSignature", Utils.ilNS));
            e.RemoveOnMethod = MethodAdapter.Instance.GetMethod(
                nav.GetAttribute("removeOnMethodSignature", Utils.ilNS));
            if (Utils.IsAttributePresent(nav, "raiseMethodSignature")) {
                e.RaiseMethod = MethodAdapter.Instance.GetMethod(
                    nav.GetAttribute("raiseMethodSignature", Utils.ilNS));
            }

            #region EventAttributes reading

            EventAttributes attrs = new EventAttributes();

            // Generals attributes
            if (Utils.IsAttributeTrue(nav, "isSpecialName")) {
                attrs |= EventAttributes.SpecialName;
            }

            // Runtime reserved attributes
            if (Utils.IsAttributeTrue(nav, "isRTSpecialName")) {
                attrs |= EventAttributes.ReservedMask &
                    EventAttributes.RTSpecialName;
            }

            e.Attributes = attrs;

            #endregion
        }
    }
}
