/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Evain Jb    (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

	using System;
	using System.Xml;
	using System.Xml.XPath;

    using Rail.MSIL;
    using Rail.Reflect;

	internal sealed class ExceptionAdapter : XmlAdapter {

		#region singleton implementation

        private ExceptionAdapter() : base("Exception")  {}
		public static readonly ExceptionAdapter Instance = 
            new ExceptionAdapter();

		#endregion

        public void ToXml(CodeNodeException exc, XmlWriter writer ) {
            writer.WriteStartElement(TagName, Utils.ilNS);

            writer.WriteAttributeString("type", Utils.ilNS,
                exc.Type.ToString().ToLower());

            if (exc.Type == CodeNodeExceptionType.Catch) {
                writer.WriteAttributeString("catchType", Utils.ilNS,
                    Utils.GetFullyQualifiedName(exc.CatchType));
            }

            CodeNode node = exc.First;
            while (node != null) {
                if (node is CodeNodeInstruction) {
                    InstructionAdapter.Instance.ToXml(
                        (CodeNodeInstruction)node, writer);
                } else if (node is CodeNodeException) {
                    ExceptionAdapter.Instance.ToXml(
                        (CodeNodeException)node, writer);
                }
                node = node.Next;
            }
            writer.WriteEndElement();
        }


        public void FromXml(MethodBody mb, CodeNodeException ex) {
            CodeTreeXml tree = (CodeTreeXml)ex.Tree;
            if (ex.Type == CodeNodeExceptionType.Try) {
                ExceptionBlock eb = new ExceptionBlock(
                    tree.GetInstruction(ex.StartOffset),
                    tree.GetInstruction(ex.EndOffset));
                mb.ExceptionTable.Add(eb);
            } else {
                CodeNodeException tr = ex.GetTryNode();
                for (int i = 0 ; i < mb.ExceptionTable.Count() ; i++) {
                    ExceptionBlock eb = mb.ExceptionTable[i];
                    if (eb.StartOfProtectedScope == tree.GetInstruction(
                            tr.StartOffset) &&
                        eb.EndOfProtectedScope == tree.GetInstruction(
                            tr.EndOffset)) {
                        
                        ExceptionHandler eh = null;
                        switch (ex.Type) {
                            case CodeNodeExceptionType.Catch :
                                eh = new ExceptionHandler(
                                    tree.GetInstruction(ex.StartOffset),
                                    tree.GetInstruction(ex.EndOffset),
                                    ex.CatchType);
                                eb.AddCatchHandler(eh);
                                break;
                            case CodeNodeExceptionType.Finally :
                                eh = new ExceptionHandler(
                                    ExceptionHandlerType.Finally,
                                    tree.GetInstruction(ex.StartOffset),
                                    tree.GetInstruction(ex.EndOffset));
                                eb.FinallyHandler = eh;
                                break;
                            case CodeNodeExceptionType.Fault :
                                eh = new ExceptionHandler(
                                    ExceptionHandlerType.Fault,
                                    tree.GetInstruction(ex.StartOffset),
                                    tree.GetInstruction(ex.EndOffset));
                                eb.FaultHandler = eh;
                                break;
                        }
                        break;
                    }
                }
            }

            for (CodeNode node = ex.First ; node != null ; node = node.Next) {
                if (node is CodeNodeInstruction) {
                    InstructionAdapter.Instance.FromXml(mb,
                        (CodeNodeInstruction)node);
                } else if (node is CodeNodeException) {
                    ExceptionAdapter.Instance.FromXml(mb,
                        (CodeNodeException)node);
                }
            }
        }
    }
}
