/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Thomas Gil (thomas.gil@dotnetguru.org)
 *   Evain Jb   (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

    using System;
    using System.Collections;
    using System.Xml;
    using System.Xml.XPath;

    using Rail.Reflect;

    internal sealed class ModuleAdapter : XmlAdapter {

        #region singleton implementation

        private ModuleAdapter() : base("Module")  {
            m_modules = new Hashtable();
        }
        public static readonly ModuleAdapter Instance = 
            new ModuleAdapter();

        #endregion

        private IDictionary m_modules;

        public IDictionary Modules {
            get { return m_modules; }
        }

        public void ToXml(RModule m, XmlWriter writer) {
            writer.WriteStartElement(TagName, Utils.ilNS);
            writer.WriteAttributeString("name", Utils.ilNS, m.Name);
            writer.WriteAttributeString("fullName", Utils.ilNS,
                m.FullyQualifiedName);

            foreach (RCustomAttrib attr in m.GetCustomAttributes(false)) {
                AttributeAdapter.Instance.ToXml(attr, writer);
            }                                                                        

            foreach (RTypeDef t in m.GetTypes()) {
                TypeAdapter.Instance.ToXml(t, writer);
            }

            writer.WriteEndElement();
        }

        public void FromXml(RAssemblyDef asm, XPathNavigator nav) {
            RModuleDef mod = asm.DefineModuleDef(
                nav.GetAttribute("name", Utils.ilNS),
                nav.GetAttribute("fullName", Utils.ilNS));

            m_modules[mod.FullyQualifiedName] = mod;

            XPathNodeIterator it = nav.Select(
                AttributeAdapter.Instance.TagName);
            while (it.MoveNext()) {
                AttributeAdapter.Instance.FromXml(mod, it.Current);
            }

            it = nav.Select(TypeAdapter.Instance.TagName);
            while (it.MoveNext()) {
                TypeAdapter.Instance.FromXml(mod, it.Current);
            }
        }

        public RModuleDef GetModule(string fullName) {
			#region DEBUG
#if DEBUG
			Logger.Log("GetModule({0})", fullName);
#endif 
			#endregion
			return (RModuleDef)m_modules[fullName];
        }
    }
}
