/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Evain Jb   (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

    using System;
    using System.Runtime.InteropServices;
    using System.Xml;
    using System.Xml.XPath;

    using Rail.Reflect;

    internal sealed class PInvokeAdapter : XmlAdapter {

        #region singleton implementation

        private PInvokeAdapter() : base("PInvoke")  {}
        public static readonly PInvokeAdapter Instance = 
            new PInvokeAdapter();

        #endregion

        public void ToXml(RPInvokeInfo pi, XmlWriter writer) {
            writer.WriteStartElement(TagName, Utils.ilNS);

            writer.WriteAttributeString("dllName", Utils.ilNS, pi.DLLName);
            writer.WriteAttributeString("entryName", Utils.ilNS, pi.EntryName);

            #region CallingConvention writing

            bool b;

            b = (pi.NativeCallingConvention & CallingConvention.Winapi) ==
                CallingConvention.Winapi;
            writer.WriteAttributeString("isWinapi", Utils.ilNS,
                b.ToString());
            b = (pi.NativeCallingConvention & CallingConvention.Cdecl) ==
                CallingConvention.Cdecl;
            writer.WriteAttributeString("isCdecl", Utils.ilNS,
                b.ToString());
            b = (pi.NativeCallingConvention & CallingConvention.StdCall) ==
                CallingConvention.StdCall;
            writer.WriteAttributeString("isStdCall", Utils.ilNS,
                b.ToString());
            b = (pi.NativeCallingConvention & CallingConvention.ThisCall) ==
                CallingConvention.ThisCall;
            writer.WriteAttributeString("isThisCall", Utils.ilNS,
                b.ToString());
            b = (pi.NativeCallingConvention & CallingConvention.FastCall) ==
                CallingConvention.FastCall;
            writer.WriteAttributeString("isFastCall", Utils.ilNS,
                b.ToString());

            #endregion

            #region CharSet writing

            b = (pi.NativeCharSet & CharSet.None) == CharSet.None;
            writer.WriteAttributeString("isNone", Utils.ilNS, b.ToString());
            b = (pi.NativeCharSet & CharSet.Ansi) == CharSet.Ansi;
            writer.WriteAttributeString("isAnsi", Utils.ilNS, b.ToString());
            b = (pi.NativeCharSet & CharSet.Unicode) == CharSet.Unicode;
            writer.WriteAttributeString("isUnicode", Utils.ilNS, b.ToString());
            b = (pi.NativeCharSet & CharSet.Auto) == CharSet.Auto;
            writer.WriteAttributeString("isAuto", Utils.ilNS, b.ToString());
            
            #endregion

            writer.WriteEndElement();
        }

        public void FromXml(RMethodDef m, XPathNavigator nav) {

            CallingConvention cc = new CallingConvention();
            CharSet cs = new CharSet();

            #region CallingConvention reading

            if (Utils.IsAttributeTrue(nav, "isWinapi")) {
                cc |= CallingConvention.Winapi;
            }
            if (Utils.IsAttributeTrue(nav, "isCdecl")) {
                cc |= CallingConvention.Cdecl;
            }
            if (Utils.IsAttributeTrue(nav, "isStdCall")) {
                cc |= CallingConvention.StdCall;
            }
            if (Utils.IsAttributeTrue(nav, "isThisCall")) {
                cc |= CallingConvention.ThisCall;
            }
            if (Utils.IsAttributeTrue(nav, "isFastCall")) {
                cc |= CallingConvention.FastCall;
            }

            #endregion

            #region CharSet reading

            if (Utils.IsAttributeTrue(nav, "isNone")) {
                cs |= CharSet.None;
            }
            if (Utils.IsAttributeTrue(nav, "isAnsi")) {
                cs |= CharSet.Ansi;
            }
            if (Utils.IsAttributeTrue(nav, "isUnicode")) {
                cs |= CharSet.Unicode;
            }
            if (Utils.IsAttributeTrue(nav, "isAuto")) {
                cs |= CharSet.Auto;
            }

            #endregion

            m.DefinePInvokeInfo(
                nav.GetAttribute("dllName", Utils.ilNS),
                nav.GetAttribute("entryName", Utils.ilNS),
                cc, cs);
        }
    }
}
