/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Thomas Gil (thomas.gil@dotnetguru.org)
 *   Evain Jb   (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

    using System;
    using System.Reflection;
    using System.Xml;
    using System.Xml.XPath;

    using Rail.Reflect;

    internal sealed class ParameterAdapter : XmlAdapter {

        #region singleton implementation

        private ParameterAdapter() : base("Parameter") {}
        public static readonly ParameterAdapter Instance = 
            new ParameterAdapter();

        #endregion

		public void ToXml(RParameter p, XmlWriter writer) {			
			writer.WriteStartElement(TagName, Utils.ilNS);

			writer.WriteAttributeString("name", Utils.ilNS, p.Name);

            writer.WriteAttributeString("type", Utils.ilNS,
                Utils.GetFullyQualifiedName(p.ParameterType));

			writer.WriteAttributeString("index", Utils.ilNS,
                p.Sequence.ToString());

            #region ParameterAttributes writing

            bool b;

            // Special Flag attributes
            b = (p.Attributes & ParameterAttributes.In) ==
                ParameterAttributes.In;
            writer.WriteAttributeString("isIn", Utils.ilNS,
                b.ToString());
            b = (p.Attributes & ParameterAttributes.Out) == 
                ParameterAttributes.Out;
            writer.WriteAttributeString("isOut", Utils.ilNS,
                b.ToString());
            b = (p.Attributes & ParameterAttributes.Lcid) ==
                ParameterAttributes.Lcid;
            writer.WriteAttributeString("isLcid", Utils.ilNS,
                b.ToString());
            b = (p.Attributes & ParameterAttributes.Retval) ==
                ParameterAttributes.Retval;
            writer.WriteAttributeString("isRetval", Utils.ilNS,
                b.ToString());
            b = (p.Attributes & ParameterAttributes.Optional) ==
                ParameterAttributes.Optional;
            writer.WriteAttributeString("isOptional", Utils.ilNS,
                b.ToString());
            b = p.Attributes == ParameterAttributes.None;
            writer.WriteAttributeString("isNone", Utils.ilNS,
                b.ToString());

            // Runtime reserved attributes
            int mask = (int)(p.Attributes & ParameterAttributes.ReservedMask);
            b = mask == (int)ParameterAttributes.HasDefault;
            writer.WriteAttributeString("isHasDefault", Utils.ilNS,
                b.ToString());
            b = mask == (int)ParameterAttributes.HasFieldMarshal;
            writer.WriteAttributeString("isHasFieldMarshal", Utils.ilNS,
                b.ToString());
            b = mask == (int)ParameterAttributes.Reserved3;
            writer.WriteAttributeString("isReserved3", Utils.ilNS,
                b.ToString());
            b = mask == (int)ParameterAttributes.Reserved4;
            writer.WriteAttributeString("isReserved4", Utils.ilNS,
                b.ToString());

            #endregion

			foreach (RCustomAttrib attr in p.GetCustomAttributes(false)) {
				AttributeAdapter.Instance.ToXml(attr, writer);
			}

			writer.WriteEndElement();
		}

        public RParameter[] FromXml(XPathNodeIterator it) {
            RParameter[] parameters = new RParameter[it.Count];
            for (int i = 0 ; it.MoveNext() ; i++) {
                parameters[i] = FromXml(it.Current);
            }
            return parameters;
        }

        private RParameter FromXml(XPathNavigator nav) {
            RParameter param = new RParameter(
                int.Parse(nav.GetAttribute("index", Utils.ilNS)),
                TypeAdapter.Instance.GetType(
                    nav.GetAttribute("type", Utils.ilNS)));

			#region Get parameter name by Teddy 2005/09/27

			param.Name = nav.GetAttribute("name", Utils.ilNS);

			#endregion
            
            #region ParameterAttributes reading

            ParameterAttributes attrs = new ParameterAttributes();

            // Special Flag attributes
            if (Utils.IsAttributeTrue(nav, "isIn")) {
                attrs |= ParameterAttributes.In;
            }
            if (Utils.IsAttributeTrue(nav, "isOut")) {
                attrs |= ParameterAttributes.Out;
            }
            if (Utils.IsAttributeTrue(nav, "isLcid")) {
                attrs |= ParameterAttributes.Lcid;
            }
            if (Utils.IsAttributeTrue(nav, "isRetval")) {
                attrs |= ParameterAttributes.Retval;
            }
            if (Utils.IsAttributeTrue(nav, "isOptional")) {
                attrs |= ParameterAttributes.Optional;
            }

            // Runtime reserved attributes
            if (Utils.IsAttributeTrue(nav, "isHasDefault")) {
                attrs |= ParameterAttributes.ReservedMask &
                    ParameterAttributes.HasDefault;
            }
            if (Utils.IsAttributeTrue(nav, "isHasFieldMarshal")) {
                attrs |= ParameterAttributes.ReservedMask &
                    ParameterAttributes.HasFieldMarshal;
            }
            if (Utils.IsAttributeTrue(nav, "isReserved3")) {
                attrs |= ParameterAttributes.ReservedMask &
                    ParameterAttributes.Reserved3;
            }
            if (Utils.IsAttributeTrue(nav, "isReserved4")) {
                attrs |= ParameterAttributes.ReservedMask &
                    ParameterAttributes.Reserved4;
            }

            param.Attributes = attrs;

            #endregion

            return param;
        }
	}
}