/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Evain Jb   (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

    using System;
    using System.Reflection;
    using System.Xml;
    using System.Xml.XPath;

    using Rail.MSIL;
    using Rail.Reflect;

    internal sealed class ResourceAdapter : XmlAdapter {

        #region singleton implementation

        private ResourceAdapter() : base("Resource")  {}
        public static readonly ResourceAdapter Instance = 
            new ResourceAdapter();

        #endregion

        public void ToXml(RResource r, XmlWriter writer) {
            writer.WriteStartElement(TagName, Utils.ilNS);
            
            writer.WriteAttributeString("name", Utils.ilNS, r.Name);
            writer.WriteAttributeString("fileName", Utils.ilNS, r.FileName);

            #region ResourceAttributes writing

            bool b;
            
            b = (r.Attributes & ResourceAttributes.Public) ==
                ResourceAttributes.Public;
            writer.WriteAttributeString("isPublic", Utils.ilNS,
                b.ToString());
            b = (r.Attributes & ResourceAttributes.Private) ==
                ResourceAttributes.Private;
            writer.WriteAttributeString("isPrivate", Utils.ilNS,
                b.ToString());

            #endregion

            writer.WriteStartAttribute("data", Utils.ilNS);
            writer.WriteBase64(r.Data, 0, r.Data.Length);
            writer.WriteEndAttribute();

            writer.WriteEndElement();
        }

        public void FromXml(RAssemblyDef asm, XPathNavigator nav) {
            
            string name = nav.GetAttribute("name", Utils.ilNS);
            string fileName = nav.GetAttribute("fileName", Utils.ilNS);

            byte[] data = Convert.FromBase64String(nav.GetAttribute(
                "data", Utils.ilNS));

            #region ResourceAttributes reading

            ResourceAttributes attrs = new ResourceAttributes();

            if (Utils.IsAttributeTrue(nav, "isPublic")) {
                attrs |= ResourceAttributes.Public;
            }
            if (Utils.IsAttributeTrue(nav, "isPrivate")) {
                attrs |= ResourceAttributes.Private;
            }

            #endregion

            asm.DefineResource(name, fileName, data, attrs);
        }

    }
}
