/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Thomas Gil (thomas.gil@dotnetguru.org)
 *   Evain Jb   (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

    using System;
    using System.Xml;
    using System.Xml.XPath;

    using Rail.MSIL;
    using Rail.Reflect;

    internal sealed class ReturnTypeAdapter : XmlAdapter {

        #region singleton implementation

        private ReturnTypeAdapter() : base("ReturnType")  {}
        public static readonly ReturnTypeAdapter Instance = 
            new ReturnTypeAdapter();

        #endregion

        public void ToXml(RReturnType ret, XmlWriter writer) {
            writer.WriteStartElement(TagName);
            writer.WriteAttributeString("type", Utils.GetFullyQualifiedName(
                ret.ReturnType));

            foreach (RCustomAttrib attr in ret.GetCustomAttributes(false)) {
                AttributeAdapter.Instance.ToXml(attr, writer);
            }

            writer.WriteEndElement();
        }

        public void FromXml(RMethodDef meth, XPathNavigator nav) {
            RReturnType ret = new RReturnType(
                TypeAdapter.Instance.GetType(nav.GetAttribute(
                    "type", Utils.ilNS)));

            XPathNodeIterator it = nav.Select(AttributeAdapter.Instance.TagName);
            while (it.MoveNext()) {
                AttributeAdapter.Instance.FromXml(ret, it.Current);
            }

            meth.ReturnType = ret;
        }
    }
}
