/*
 * Copyright (c) 2004 DotNetGuru and the individuals listed
 * on the ChangeLog entries.
 *
 * Authors :
 *   Evain Jb   (jb@evain.net)
 *
 * This is a free software distributed under a BSD license
 * See LICENSE.BSD file for more details
 *
 *****************************************************************************/

namespace Ilml.Adapters {

    using System;
    using System.Collections;
    using System.Xml;
    using System.Xml.XPath;

    using Rail.Reflect;

    internal sealed class VersionAdapter : XmlAdapter {

        #region singleton implementation

        private VersionAdapter() : base("Version")  {}
        public static readonly VersionAdapter Instance = 
            new VersionAdapter();

        #endregion

        public void ToXml(RVersion vers, XmlWriter writer) {
            writer.WriteStartElement(TagName, Utils.ilNS);

            writer.WriteAttributeString("major", Utils.ilNS,
                vers.Major.ToString());
            writer.WriteAttributeString("minor", Utils.ilNS,
                vers.Minor.ToString());
            writer.WriteAttributeString("build", Utils.ilNS,
                vers.Build.ToString());
            writer.WriteAttributeString("revision", Utils.ilNS,
                vers.Revision.ToString());

            writer.WriteEndElement();
        }

        public void FromXml(RAssemblyNameDef name,
            XPathNavigator nav) {
            
            int major = int.Parse(nav.GetAttribute(
                "major", Utils.ilNS));
            int minor = int.Parse(nav.GetAttribute(
                "minor", Utils.ilNS));
            int build = int.Parse(nav.GetAttribute(
                "build", Utils.ilNS));
            int revision = int.Parse(nav.GetAttribute(
                "revision", Utils.ilNS));

            name.Version = new RVersion(major, minor, build, revision);
        }
    }
}
